# https://github.com/sindresorhus/weechat-notification-center
# Requires `pip install pync`

import os
import datetime
import weechat
from pync import Notifier


SCRIPT_NAME = 'notification_center'
SCRIPT_AUTHOR = 'Sindre Sorhus <sindresorhus@gmail.com>'
SCRIPT_VERSION = '1.5.2'
SCRIPT_LICENSE = 'MIT'
SCRIPT_DESC = 'Pass highlights and private messages to the macOS Notification Center'

weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', '')

WEECHAT_VERSION = weechat.info_get('version_number', '') or 0
if int(WEECHAT_VERSION) >= 0x03020000:
	WEECHAT_ICON = os.path.join(weechat.info_get('weechat_config_dir', ''), 'weechat.png')
else:
	WEECHAT_ICON = os.path.join(weechat.info_get('weechat_dir', ''), 'weechat.png')

DEFAULT_OPTIONS = {
	'show_highlights': 'on',
	'show_private_message': 'on',
	'show_message_text': 'on',
	'sound': 'off',
	'sound_name': 'Pong',
	'activate_bundle_id': 'com.apple.Terminal',
	'ignore_old_messages': 'off',
	'ignore_current_buffer_messages': 'off',
	'channels': '',
	'tags': '',
}

for key, val in DEFAULT_OPTIONS.items():
	if not weechat.config_is_set_plugin(key):
		weechat.config_set_plugin(key, val)

weechat.hook_print('', 'irc_privmsg,' + weechat.config_get_plugin('tags'), '', 1, 'notify', '')

def notify(data, buffer, date, tags, displayed, highlight, prefix, message):
	# Ignore if it's yourself
	own_nick = weechat.buffer_get_string(buffer, 'localvar_nick')
	if prefix == own_nick or prefix == ('@%s' % own_nick):
		return weechat.WEECHAT_RC_OK

	# Ignore messages from the current buffer
	if weechat.config_get_plugin('ignore_current_buffer_messages') == 'on' and buffer == weechat.current_buffer():
		return weechat.WEECHAT_RC_OK

	# Ignore messages older than the configured theshold (such as ZNC logs) if enabled
	if weechat.config_get_plugin('ignore_old_messages') == 'on':
		message_time = datetime.datetime.utcfromtimestamp(int(date))
		now_time = datetime.datetime.utcnow()

		# Ignore if the message is greater than 5 seconds old
		if (now_time - message_time).seconds > 5:
			return weechat.WEECHAT_RC_OK

	# Passing `None` or `''` still plays the default sound so we pass a lambda instead
	sound = weechat.config_get_plugin('sound_name') if weechat.config_get_plugin('sound') == 'on' else lambda:_
	activate_bundle_id = weechat.config_get_plugin('activate_bundle_id')

	channel_allow_list = []
	if weechat.config_get_plugin('channels') != "":
		channel_allow_list = weechat.config_get_plugin('channels').split(',')
	channel = weechat.buffer_get_string(buffer, 'localvar_channel')

	if channel in channel_allow_list:
		if weechat.config_get_plugin('show_message_text') == 'on':
			Notifier.notify(message, title='%s %s' % (prefix, channel), sound=sound, appIcon=WEECHAT_ICON, activate=activate_bundle_id)
		else:
			Notifier.notify('In %s by %s' % (channel, prefix), title='Channel Activity', sound=sound, appIcon=WEECHAT_ICON, activate=activate_bundle_id)
	elif weechat.config_get_plugin('show_highlights') == 'on' and int(highlight):
		if weechat.config_get_plugin('show_message_text') == 'on':
			Notifier.notify(message, title='%s %s' % (prefix, channel), sound=sound, appIcon=WEECHAT_ICON, activate=activate_bundle_id)
		else:
			Notifier.notify('In %s by %s' % (channel, prefix), title='Highlighted Message', sound=sound, appIcon=WEECHAT_ICON, activate=activate_bundle_id)
	elif weechat.config_get_plugin('show_private_message') == 'on' and 'irc_privmsg' in tags and 'notify_private' in tags:
		if weechat.config_get_plugin('show_message_text') == 'on':
			Notifier.notify(message, title='%s [private]' % prefix, sound=sound, appIcon=WEECHAT_ICON, activate=activate_bundle_id)
		else:
			Notifier.notify('From %s' % prefix, title='Private Message', sound=sound, appIcon=WEECHAT_ICON, activate=activate_bundle_id)
	return weechat.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 p3lim <weechat@p3lim.net>
#
# https://github.com/p3lim/weechat-detach-away
#
# Changelog:
# Ver: 0.1.1 Python3 support by Antonin Skala skala.antonin@gmail.com 3.2019
# Ver: 0.1.2 Support Python 2 and 3 by Antonin Skala skala.antonin@gmail.com 3.2019

try:
    import weechat
except ImportError:
    from sys import exit
    print('This script has to run under WeeChat (https://weechat.org/).')
    exit(1)

import sys

if sys.version_info[0] > 2:
    from urllib.parse import urlencode
else:
    from urllib import urlencode

SCRIPT_NAME = 'detach_away'
SCRIPT_AUTHOR = 'p3lim'
SCRIPT_VERSION = '0.1.2'
SCRIPT_LICENSE = 'MIT'
SCRIPT_DESC = 'Automatically sets away message based on number of relays connected'

SETTINGS = {
    'message': (
        'I am away',
        'away message'),
    'debugging': (
        'off',
        'debug flag'),
}

num_relays = 0

def DEBUG():
    return weechat.config_get_plugin('debug') == 'on'

def set_away(is_away, message=''):
    if is_away:
        message = weechat.config_get_plugin('message')

    weechat.command('', '/away -all ' + message)

def relay_connected(data, signal, signal_data):
    global num_relays

    if DEBUG():
        weechat.prnt('', 'DETACH_AWAY: last #relays: ' + str(num_relays))

    if int(num_relays) == 0:
        set_away(False)

    num_relays = weechat.info_get('relay_client_count', 'connected')
    return weechat.WEECHAT_RC_OK

def relay_disconnected(data, signal, signal_data):
    global num_relays

    if DEBUG():
        weechat.prnt('', 'DETACH_AWAY: last #relays: ' + str(num_relays))

    if int(num_relays) > 0:
        set_away(True)

    num_relays = weechat.info_get('relay_client_count', 'connected')
    return weechat.WEECHAT_RC_OK

# register plugin
weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', '')

# register for relay status
weechat.hook_signal('relay_client_connected', 'relay_connected', '')
weechat.hook_signal('relay_client_disconnected', 'relay_disconnected', '')

# register configuration defaults
for option, value in SETTINGS.items():
    if not weechat.config_is_set_plugin(option):
        weechat.config_set_plugin(option, value[0])
    weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 by Oscar Morante <oscar@morante.eu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# This script uses fribidi library to display rtl text properly

import weechat
from pyfribidi import *

SCRIPT_NAME    = "biditext"
SCRIPT_AUTHOR  = "Oscar Morante <oscar@morante.eu>"
SCRIPT_VERSION = "1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Use fribidi to handle RTL text"


def biditext_cb(data, modifier, modifier_data, line):
    return log2vis(line, LTR)


if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME,
                        SCRIPT_AUTHOR,
                        SCRIPT_VERSION,
                        SCRIPT_LICENSE,
                        SCRIPT_DESC, "", ""):
        weechat.hook_modifier('weechat_print', 'biditext_cb', '')




# -*- coding: utf-8 -*-
# Copyright (c) 2009 by Wraithan <XWraithanX@gmail.com>
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
#
# 2009-06-18, Wraithan <XWraithanX@gmail.com>
#   version 0.1: initial release.
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
''' Kilosecond display. 

Usage: /ks to display the current time in kiloseconds,
Also makes availbe [kiloseconds] for your status bar. 
" weechat.bar.status.items".
'''

import weechat
import time

SCRIPT_NAME    = "kiloseconds"
SCRIPT_AUTHOR  = "Wraithan <XWraithanX@gmail.com>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Script to display the current time in kiloseconds in various ways."

SCRIPT_COMMAND = "ks"

def getKiloseconds():
    """Get current time in kiloseconds"""
    tm = time.localtime()
    return (tm.tm_hour*3600+tm.tm_min*60+tm.tm_sec)/1000.0 
    

def kiloseconds_cmd(data, buffer, args):
    """Callback for /ks command"""
    weechat.command(buffer,"The current time is " + str(getKiloseconds()) + " KS")
    return weechat.WEECHAT_RC_OK

def kiloseconds_cb(data, buffer, args):
    """Callback for the bar item"""
    return "%06.3f" % getKiloseconds()

def kiloseconds_update(data,cals):
    """Update the bar item"""
    weechat.bar_item_update('kiloseconds')
    return weechat.WEECHAT_RC_OK

if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, '', ''):

    weechat.hook_command(SCRIPT_COMMAND,
                         "Display current time in kiloseconds",
                         "",
                         "Add 'kiloseconds' to any bar to have it show the time in kiloseconds.",
                         "",
                         "kiloseconds_cmd", "")

    weechat.bar_item_new('kiloseconds', 'kiloseconds_cb', '')
    weechat.hook_timer(1000,1,0,'kiloseconds_update','')




# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 by Simmo Saan <simmo.saan@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# History:
#
# 2021-11-06, Sébastien Helleu <flashcode@flashtux.org>
#   version 0.11: make script compatible with WeeChat >= 3.4
#                 (new parameters in function hdata_search)
# 2019-06-07, Trygve Aaberge <trygveaa@gmail.com>
#   version 0.10: remove newlines from command completion
# 2015-10-04, Simmo Saan <simmo.saan@gmail.com>
#   version 0.9: fix text search imitation in filter
# 2015-08-27, Simmo Saan <simmo.saan@gmail.com>
#   version 0.8: add documentation
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.7: mute filter add/del
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.6: imitate search settings in filter
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.5: option for bar item text
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.4: option for default state
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.3: allow toggling during search
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.2: add bar item for indication
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.1: initial script
#

"""
Filter buffers automatically while searching them
"""

from __future__ import print_function

SCRIPT_NAME = "grep_filter"
SCRIPT_AUTHOR = "Simmo Saan <simmo.saan@gmail.com>"
SCRIPT_VERSION = "0.11"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Filter buffers automatically while searching them"

SCRIPT_REPO = "https://github.com/sim642/grep_filter"

SCRIPT_COMMAND = SCRIPT_NAME
SCRIPT_BAR_ITEM = SCRIPT_NAME
SCRIPT_LOCALVAR = SCRIPT_NAME

IMPORT_OK = True

try:
    import weechat
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org/")
    IMPORT_OK = False

import re  # re.escape

SETTINGS = {
    "enable": ("off", "enable automatically start filtering when searching"),
    "bar_item": ("grep", "text to show in bar item when filtering"),
}

KEYS = {"ctrl-G": "/%s toggle" % SCRIPT_COMMAND}


def get_merged_buffers(ptr):
    """
    Get a list of buffers which are merged with "ptr".
    """

    weechat_version = int(weechat.info_get("version_number", "") or 0)

    hdata = weechat.hdata_get("buffer")
    buffers = weechat.hdata_get_list(hdata, "gui_buffers")
    if weechat_version >= 0x03040000:
        buffer = weechat.hdata_search(
            hdata,
            buffers,
            "${buffer.number} == ${value}",
            {},
            {"value": str(weechat.hdata_integer(hdata, ptr, "number"))},
            {},
            1,
        )
    else:
        buffer = weechat.hdata_search(
            hdata,
            buffers,
            "${buffer.number} == %i"
            % weechat.hdata_integer(hdata, ptr, "number"),
            1,
        )
    nbuffer = weechat.hdata_move(hdata, buffer, 1)

    ret = []
    while buffer:
        ret.append(weechat.hdata_string(hdata, buffer, "full_name"))

        if weechat.hdata_integer(
            hdata, buffer, "number"
        ) == weechat.hdata_integer(hdata, nbuffer, "number"):
            buffer = nbuffer
            nbuffer = weechat.hdata_move(hdata, nbuffer, 1)
        else:
            buffer = None

    return ret


def filter_exists(name):
    """
    Check whether a filter named "name" exists.
    """

    weechat_version = int(weechat.info_get("version_number", "") or 0)

    hdata = weechat.hdata_get("filter")
    filters = weechat.hdata_get_list(hdata, "gui_filters")
    if weechat_version >= 0x03040000:
        filter = weechat.hdata_search(
            hdata,
            filters,
            "${filter.name} == ${name}",
            {},
            {"name": name},
            {},
            1,
        )
    else:
        filter = weechat.hdata_search(
            hdata,
            filters,
            "${filter.name} == %s" % name,
            1,
        )

    return bool(filter)


def filter_del(name):
    """
    Delete a filter named "name".
    """

    weechat.command(
        weechat.buffer_search_main(), "/mute filter del %s" % name
    )


def filter_addreplace(name, buffers, tags, regex):
    """
    Add (or replace if already exists) a filter named "name" with specified argumets.
    """

    if filter_exists(name):
        filter_del(name)

    weechat.command(
        weechat.buffer_search_main(),
        "/mute filter add %s %s %s %s" % (name, buffers, tags, regex),
    )


def buffer_searching(buffer):
    """
    Check whether "buffer" is in search mode.
    """

    hdata = weechat.hdata_get("buffer")

    return bool(weechat.hdata_integer(hdata, buffer, "text_search"))


def buffer_filtering(buffer):
    """
    Check whether "buffer" should be filtered.
    """

    local = weechat.buffer_get_string(buffer, "localvar_%s" % SCRIPT_LOCALVAR)
    return {"": None, "0": False, "1": True}[local]


def buffer_build_regex(buffer):
    """
    Build a regex according to "buffer"'s search settings.
    """

    hdata = weechat.hdata_get("buffer")
    input = weechat.hdata_string(hdata, buffer, "input_buffer")
    exact = weechat.hdata_integer(hdata, buffer, "text_search_exact")
    where = weechat.hdata_integer(hdata, buffer, "text_search_where")
    regex = weechat.hdata_integer(hdata, buffer, "text_search_regex")

    if not regex:
        input = re.escape(input)

    if exact:
        input = "(?-i)%s" % input

    filter_regex = None
    if where == 1:  # message
        filter_regex = input
    elif where == 2:  # prefix
        filter_regex = "%s\\t" % input
    else:  # prefix | message
        filter_regex = input  # TODO: impossible with current filter regex

    return "!%s" % filter_regex


def buffer_update(buffer):
    """
    Refresh filtering in "buffer" by updating (or removing) the filter and update the bar item.
    """

    hdata = weechat.hdata_get("buffer")

    buffers = ",".join(get_merged_buffers(buffer))
    name = "%s_%s" % (SCRIPT_NAME, buffers)

    if buffer_searching(buffer):
        if buffer_filtering(buffer):
            filter_addreplace(name, buffers, "*", buffer_build_regex(buffer))
        elif not buffer_filtering(buffer) and filter_exists(name):
            filter_del(name)
    elif filter_exists(name):
        filter_del(name)

    where = weechat.hdata_integer(hdata, buffer, "text_search_where")
    weechat.buffer_set(
        buffer,
        "localvar_set_%s_warn" % SCRIPT_LOCALVAR,
        "1" if where == 3 else "0",
    )  # warn about incorrect filter

    weechat.bar_item_update(SCRIPT_BAR_ITEM)


def input_search_cb(data, signal, buffer):
    """
    Handle "input_search" signal.
    """

    if buffer_searching(buffer) and buffer_filtering(buffer) is None:
        enable = weechat.config_string_to_boolean(
            weechat.config_get_plugin("enable")
        )
        weechat.buffer_set(
            buffer,
            "localvar_set_%s" % SCRIPT_LOCALVAR,
            "1" if enable else "0",
        )
        weechat.buffer_set(
            buffer, "localvar_set_%s_warn" % SCRIPT_LOCALVAR, "0"
        )
    elif not buffer_searching(buffer):
        weechat.buffer_set(buffer, "localvar_del_%s" % SCRIPT_LOCALVAR, "")
        weechat.buffer_set(
            buffer, "localvar_del_%s_warn" % SCRIPT_LOCALVAR, ""
        )

    buffer_update(buffer)

    return weechat.WEECHAT_RC_OK


def input_text_changed_cb(data, signal, buffer):
    """
    Handle "input_text_changed" signal.
    """

    if buffer_searching(buffer) and buffer_filtering(buffer):
        buffers = ",".join(get_merged_buffers(buffer))
        name = "%s_%s" % (SCRIPT_NAME, buffers)

        filter_addreplace(name, buffers, "*", buffer_build_regex(buffer))

    return weechat.WEECHAT_RC_OK


def command_cb(data, buffer, args):
    """
    Handle command.
    """

    if args == "enable":
        weechat.buffer_set(buffer, "localvar_set_%s" % SCRIPT_LOCALVAR, "1")
    elif args == "disable":
        weechat.buffer_set(buffer, "localvar_set_%s" % SCRIPT_LOCALVAR, "0")
    elif args == "toggle":
        weechat.buffer_set(
            buffer,
            "localvar_set_%s" % SCRIPT_LOCALVAR,
            "0" if buffer_filtering(buffer) else "1",
        )
    else:
        pass

    buffer_update(buffer)

    return weechat.WEECHAT_RC_OK


def bar_item_cb(data, item, window, buffer, extra_info):
    """
    Build the bar item's content for "buffer".
    """

    buffers = ",".join(get_merged_buffers(buffer))
    name = "%s_%s" % (SCRIPT_NAME, buffers)

    if filter_exists(name):
        warn = int(
            weechat.buffer_get_string(
                buffer, "localvar_%s_warn" % SCRIPT_LOCALVAR
            )
        )

        return "%s%s%s" % (
            weechat.color("input_text_not_found" if warn else "bar_fg"),
            weechat.config_get_plugin("bar_item"),
            weechat.color("reset"),
        )
    else:
        return ""


if __name__ == "__main__" and IMPORT_OK:
    if weechat.register(
        SCRIPT_NAME,
        SCRIPT_AUTHOR,
        SCRIPT_VERSION,
        SCRIPT_LICENSE,
        SCRIPT_DESC,
        "",
        "",
    ):
        weechat.hook_signal("input_search", "input_search_cb", "")
        weechat.hook_signal("input_text_changed", "input_text_changed_cb", "")

        weechat.hook_command(
            SCRIPT_COMMAND,
            SCRIPT_DESC,
            """enable || disable || toggle""",
            """ enable: enable {0} in current buffer
disable: disable {0} in current buffer
 toggle: toggle {0} in current buffer

By default a bind in "search" context is added to toggle with "ctrl-G".
To see {0} status during search, add "{1}" item to some bar. On default configuration you can do it with:
    /set weechat.bar.input.items "[input_prompt]+(away),[{1}],[input_search],[input_paste],input_text"

Due to technical reasons with /filter it is not possible to exactly {0} in "pre|msg" search mode, thus the bar item is shown in warning color.""".format(
                SCRIPT_NAME, SCRIPT_BAR_ITEM
            ),
            """enable || disable || toggle""",
            "command_cb",
            "",
        )

        weechat.bar_item_new("(extra)%s" % SCRIPT_BAR_ITEM, "bar_item_cb", "")

        for option, value in SETTINGS.items():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, value[0])

            weechat.config_set_desc_plugin(
                option, '%s (default: "%s")' % (value[1], value[0])
            )

        weechat.key_bind("search", KEYS)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013-2014 Germain Z. <germanosz@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Convert text to its ｆｕｌｌｗｉｄｔｈ equivalent and send it to buffer.
#


import sys
import weechat


SCRIPT_NAME = "fullwidth"
SCRIPT_AUTHOR = "GermainZ <germanosz@gmail.com>"
SCRIPT_VERSION = "0.1.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = ("Convert text to its ｆｕｌｌｗｉｄｔｈ equivalent and send it "
               "to buffer.")

PY3 = sys.version > '3'


if PY3:
    unichr = chr
    def send(buf, text):
        weechat.command(buf, "/input send {}".format(text))
else:
    def send(buf, text):
        weechat.command(buf, "/input send {}".format(text.encode("utf-8")))

def cb_fullwidth_cmd(data, buf, args):
    """Callback for ``/fullwidth``, convert and send the given text."""
    chars = []
    if not PY3:
        args = args.decode("utf-8")
    for char in list(args):
        ord_char = ord(char)
        if ord_char == 32:
            char = unichr(12288)
        elif ord_char > 32 and ord_char <= 126:
            char = unichr(ord_char + 65248)
        chars.append(char)
    send(buf, ''.join(chars))
    return weechat.WEECHAT_RC_OK


if __name__ == "__main__":
    weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                     SCRIPT_LICENSE, SCRIPT_DESC, '', '')
    weechat.hook_command("fullwidth", SCRIPT_DESC, "<text>", '', '',
                         "cb_fullwidth_cmd", '')



# -*- coding: utf-8 -*-
#
# Insert a giphy URL based on a command and search
# Use giphys random, search and translate from weechat
# Usage: /giphy search Search Term
# Usage: /giphy msg message
# Usage: /giphy random Search Term
# Usage: /gipgy Search Term
#
# History:
#
# 2018-10-14, butlerx
#   Version 1.0.2: clean up code
# 2017-04-19, butlerx
#   Version 1.0.1: remove + from message
# 2017-04-18, butlerx
#   Version 1.0.0: initial version
#

from __future__ import absolute_import

from requests import get

from weechat import WEECHAT_RC_OK, command, hook_command, register

SCRIPT_NAME = "giphy"
SCRIPT_AUTHOR = "butlerx <butlerx@redbrick.dcu.ie>"
SCRIPT_VERSION = "1.0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Insert giphy gif"


def giphy(data, buf, args):
    """ Parse args to decide what api to use """
    search_string = args.split()
    arg = search_string.pop(0)
    search_string = "+".join(search_string)
    results = (
        api_request("search", {"limit": 1, "q": search_string})
        if arg == "search"
        else api_request("translate", {"s": search_string})
        if arg == "msg"
        else api_request("random", {"tag": search_string})
        if arg == "random"
        else api_request("random", {"tag": "+".join([arg, search_string])})
    )
    command(
        buf, "giphy {} -- {}".format(search_string.replace("+", " ").strip(), results)
    )
    return WEECHAT_RC_OK


def api_request(method, params):
    """Query giphy api for search"""
    try:
        params["api_key"] = "dc6zaTOxFJmzC"
        response = get("http://api.giphy.com/v1/gifs/{}".format(method), params=params)
        data = response.json()["data"]
        data = data[0] if isinstance(data, list) else data
        return (
            data["images"]["original"]["url"]
            if "image_url" not in data
            else data["image_url"]
        )
    except TypeError:
        return "No GIF good enough"


if __name__ == "__main__":
    if register(
        SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""
    ):
        hook_command(SCRIPT_NAME, SCRIPT_DESC, "", "", "", SCRIPT_NAME, "")



#
# SPDX-FileCopyrightText: 2013-2025 Nils Görs <weechatter@arcor.de>
# SPDX-FileCopyrightText: 2017 Filip H.F. 'FiXato' Slagter <fixato+weechat@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Save and restore query buffers after /quit.
#
# Idea by lasers@freenode.#weechat
#
# 2025-10-02: nils_2 (libera.#weechat)
#       0.7 : fix a ValueError in config file, when using dcc. localvar "server", is missing for dcc buffer (reported by roughnecks)
#           : add some more DEBUG() text
#
# 2023-08-01: nils_2 (libera.#weechat)
#     0.6.1 : fix a timing problem when joining autojoin-channels (reported by thecdnhermit)
#
# 2021-05-05: Sébastien Helleu <flashcode@flashtux.org>
#       0.6 : add compatibility with XDG directories (WeeChat >= 3.2)
#
# 2018-08-08: nils_2, (freenode.#weechat)
#       0.5 : fix TypeError with python3.6
#
# 2017-04-14: nils_2 & FiXato, (freenode.#weechat)
#       0.4 : big rewrite:
#           : added extra hooks:
#           - query buffers are now also stored when opening/closing queries
#           - queries only restored on connect; no longer on every reconnect
#           : current buffer position is retained
#           : manual saving of query list (https://github.com/weechat/scripts/issues/196)
#
# 2015-02-27: nils_2, (freenode.#weechat)
#       0.3 : make script consistent with "buffer_switch_autojoin" option (idea haasn)
#
# 2013-11-07: nils_2, (freenode.#weechat)
#       0.2 : fix file not found error (reported by calcifea)
#           : make script compatible with Python 3.x
#
# 2013-07-26: nils_2, (freenode.#weechat)
#       0.1 : initial release
#
# script will create a config file (~./weechat/queryman.txt)
# format: "servername nickname" (without "")
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat,re,os

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = 'queryman'
SCRIPT_AUTHOR   = 'nils_2 <weechatter@arcor.de>'
SCRIPT_VERSION  = '0.7'
SCRIPT_LICENSE  = 'GPL'
SCRIPT_DESC     = 'save and restore query buffers after /quit and on open/close of queries'
DEBUG           = False

queryman_filename = 'queryman.txt'
servers_opening = set([])
servers_closing = set([])
stored_query_buffers_per_server = {}

# ================================[ callback ]===============================
# signal_data = buffer pointer
def buffer_closing_signal_cb(data, signal, signal_data):
    global servers_closing

    buf_type = weechat.buffer_get_string(signal_data,'localvar_type')
    # When closing a server buffer, save all buffers
    if buf_type == 'server':
        # Prevent closing private buffers on this server, from triggering saving.
        servers_closing.add(weechat.buffer_get_string(signal_data, 'localvar_server'))

        # # FIXME: This shouldn't be necessary, as buffers are already save when opened/closed
        # reset_stored_query_buffers()
        # save_stored_query_buffers_to_file()
    # Only save the query buffers
    elif buf_type == 'private':
        server_name = weechat.buffer_get_string(signal_data, 'localvar_server')
        # Don't trigger when all buffer's closing because its server buffer is closing
        if server_name not in servers_closing:
            channel_name = weechat.buffer_get_string(signal_data, 'localvar_channel')
            remove_channel_from_stored_list(server_name, channel_name)
            save_stored_query_buffers_to_file()
    return weechat.WEECHAT_RC_OK

def quit_signal_cb(data, signal, signal_data):
    reset_stored_query_buffers()
    save_stored_query_buffers_to_file()
    return weechat.WEECHAT_RC_OK

# signal_data = buffer pointer
def irc_pv_opened_cb(data, signal, signal_data):
    server_name = weechat.buffer_get_string(signal_data, 'localvar_server')
    channel_name = weechat.buffer_get_string(signal_data, 'localvar_channel')
    add_channel_to_stored_list(server_name, channel_name)
    debug_print("signal: irc_pv_opened server: %s query: %s" % (server_name, channel_name))
    save_stored_query_buffers_to_file()
    return weechat.WEECHAT_RC_OK

# signal_data = server name
def remove_server_from_servers_closing_cb(data, signal, signal_data):
    global servers_closing
    if signal_data in servers_closing:
        servers_closing.remove(signal_data)
        debug_print("signal: irc_server_closing")
    return weechat.WEECHAT_RC_OK

# signal_data = buffer pointer
def irc_server_opened_cb(data, signal, signal_data):
    global servers_opening

    server_name = weechat.buffer_get_string(signal_data, 'localvar_server')
    servers_opening.add(server_name)
    debug_print("signal: irc_server_opened %s" % server_name)
    return weechat.WEECHAT_RC_OK

# signal_data = servername
def irc_server_connected_signal_cb(data, signal, signal_data):
    global servers_opening

    # Only reopen the query buffers if the server buffer was recently opened
    if signal_data in servers_opening:
        open_stored_query_buffers_for_server(signal_data)
        servers_opening.remove(signal_data)
        debug_print("signal: irc_server_connected")

    return weechat.WEECHAT_RC_OK

# ================================[ file ]===============================
def get_filename_with_path():
    global queryman_filename
    path = weechat.info_get("weechat_data_dir", "") \
        or weechat.info_get("weechat_dir", "")
    return os.path.join(path,queryman_filename)

# ======== [ Stored Query Buffers List ] ==========
def get_stored_list_of_query_buffers():
    global stored_query_buffers_per_server

    filename = get_filename_with_path()
    stored_query_buffers_per_server = {}

    if os.path.isfile(filename):
        f = open(filename, 'r')
        for line in f:
            parts = line.strip().split()
            if len(parts) == 2:
                server_name, nick = parts
                stored_query_buffers_per_server.setdefault(server_name, set([]))
                stored_query_buffers_per_server[server_name].add(nick)
        f.close()
    else:
        debug_print('Error loading query buffer from "%s"' % filename)
    return stored_query_buffers_per_server

def remove_channel_from_stored_list(server_name, channel_name):
    global stored_query_buffers_per_server

    if server_name in stored_query_buffers_per_server and channel_name in stored_query_buffers_per_server[server_name]:
        stored_query_buffers_per_server[server_name].remove(channel_name)
        if not len(stored_query_buffers_per_server[server_name]):
            stored_query_buffers_per_server.pop(server_name, None)

def add_channel_to_stored_list(server_name, channel_name):
    global stored_query_buffers_per_server

    if server_name not in stored_query_buffers_per_server:
        stored_query_buffers_per_server[server_name] = set([])
    if channel_name not in stored_query_buffers_per_server[server_name]:
        stored_query_buffers_per_server[server_name].add(channel_name)

def open_query_buffer(server_name, nick):
    starting_buffer = weechat.current_buffer()
    noswitch = ""
    switch_autojoin = weechat.config_get("irc.look.buffer_switch_autojoin")
    if not weechat.config_boolean(switch_autojoin):
        noswitch = "-noswitch"

    debug_print("opening query buffer: %s on server %s" % (nick, server_name))
    weechat.command('','/wait 1 /query %s -server %s %s' % ( noswitch, server_name, nick ))
    weechat.buffer_set(starting_buffer, 'display', 'auto')

def open_stored_query_buffers_for_server(server_connected):
    global stored_query_buffers_per_server

    if server_connected in stored_query_buffers_per_server:
        for nick in stored_query_buffers_per_server[server_connected].copy():
            debug_print("going to open query buffer: %s on connected server %s" % (nick, server_connected))
            open_query_buffer(server_connected, nick)

def get_current_query_buffers():
    stored_query_buffers_per_server = {}

    ptr_infolist_buffer = weechat.infolist_get('buffer', '', '')
    while weechat.infolist_next(ptr_infolist_buffer):
        ptr_buffer = weechat.infolist_pointer(ptr_infolist_buffer,'pointer')

        buf_type = weechat.buffer_get_string(ptr_buffer, 'localvar_type')
        if buf_type == 'private':
            server_name = weechat.buffer_get_string(ptr_buffer, 'localvar_server')
            channel_name = weechat.buffer_get_string(ptr_buffer, 'localvar_channel')

            stored_query_buffers_per_server.setdefault(server_name, set([]))
            stored_query_buffers_per_server[server_name].add(channel_name)
    weechat.infolist_free(ptr_infolist_buffer)

    return stored_query_buffers_per_server

def reset_stored_query_buffers():
    global stored_query_buffers_per_server
    stored_query_buffers_per_server = get_current_query_buffers()


def remove_data_file():
    filename = get_filename_with_path()
    if os.path.isfile(filename):
        os.remove(filename)

def save_stored_query_buffers_to_file():
    global stored_query_buffers_per_server

    filename = get_filename_with_path()
    if len(stored_query_buffers_per_server):
        debug_print("Storing %s servers:" % len(stored_query_buffers_per_server))
        try:
            f = open(filename, 'w')
            for (server_name, channels) in stored_query_buffers_per_server.items():
                debug_print("Storing %s channels in server %s" % (len(channels), server_name))
                for channel_name in channels:
                    line = "%s %s" % (server_name,channel_name)
                    debug_print(' - %s' % line)
                    f.write("%s\n" % line)
            f.close()
        except:
            print_error('Error writing query buffer to "%s"' % filename)
            raise
    else:       # no query buffer(s). remove file
        debug_print("No stored query buffers; removing data file")
        remove_data_file()
    return

def print_error(message):
    weechat.prnt('','%s%s: %s' % (weechat.prefix('error'), SCRIPT_NAME, message))

def debug_print(message):
    if not DEBUG:
        return
    weechat.prnt('','DEBUG/%s: %s' % (SCRIPT_NAME, message))

def hook_command_cb(data, buffer, args):
    if args == "":                                                                              # no args given. quit
        return weechat.WEECHAT_RC_OK
    argv = args.strip().split(" ")
    if argv[0].lower() == 'save':
        save_stored_query_buffers_to_file()
    return weechat.WEECHAT_RC_OK

# ================================[ main ]===============================
if __name__ == '__main__':
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get('version_number', '') or 0
        if int(version) >= 0x00030700:
            weechat.hook_command(SCRIPT_NAME,SCRIPT_DESC,
            'save',
            'save : manual saving of the query list\n',
            '',
            'hook_command_cb', '')

            stored_query_buffers_per_server = get_stored_list_of_query_buffers()
            for (server_name, channels) in get_current_query_buffers().items():
                # Reopen the buffers for the channels in the servers we already have open:
                open_stored_query_buffers_for_server(server_name)

                stored_query_buffers_per_server.setdefault(server_name, set([]))
                debug_print("Already have %s queries for server %s: %s" % (len(stored_query_buffers_per_server[server_name]), server_name, ','.join(stored_query_buffers_per_server[server_name])))
                debug_print("Adding: %s" % channels)
                stored_query_buffers_per_server[server_name].update(channels)
                debug_print("Now have %s queries for server %s: %s" % (len(stored_query_buffers_per_server[server_name]), server_name, ','.join(stored_query_buffers_per_server[server_name])))
            save_stored_query_buffers_to_file()
            weechat.hook_signal('quit', 'quit_signal_cb', '')
#            weechat.hook_signal('relay_client_disconnected', 'quit_signal_cb', '')
#            weechat.hook_signal('relay_client_connected', 'irc_server_connected_signal_cb', '')
            weechat.hook_signal('irc_server_opened', 'irc_server_opened_cb', '')
            weechat.hook_signal('irc_server_connected', 'irc_server_connected_signal_cb','')
            weechat.hook_signal('irc_server_disconnected', 'remove_server_from_servers_closing_cb', '')

            # TODO: make these triggers optional?
            weechat.hook_signal('irc_pv_opened', 'irc_pv_opened_cb', '')
            weechat.hook_signal('buffer_closing', 'buffer_closing_signal_cb', '')



# coding: utf-8
#
# Copyright (c) 2012-2018 by nesthib <nesthib@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# This script checks every message before it is sent and blocks messages which
# correspond to misformatted commands (e.g. " /msg NickServ…") to avoid the
# unfortunate discosure of personnal informations.
#
# 2022-11-11: Kevin Morris <kevr@cost.org>
#        0.3: remove leading whitespaces in all situations
# 2018-06-07: nils_2@freenode.#weechat
#        0.2: make script compatible with Python 3.x
# 2012-03-07: nesthib <nesthib@gmail.com>
#        0.1: initial release

try:
    import weechat
except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org")
    quit()

import re

name = "unwanted_msg"
author = "nesthib <nesthib@gmail.com>"
version = "0.3"
license = "GPL"
description = "Avoid sending misformatted messages"
shutdown_function = ""
charset = ""

config = {
    "left_delimiter": "[",
    "right_delimiter": "]"
}

weechat.register(name, author, version, license,
                 description, shutdown_function, charset)


def raw_command_cb(data, buffer, args):
    """ Implementation of the /raw command.

    /raw sends arguments given as verbatim as possible to the buffer.
    Commands themselves remove whitespace found between the command and
    any arguments, and so, a special delimiter is required to decipher
    what the user intends when formatting inputs with included
    whitespace: [ some message].


    Example output:
        /raw  hello
            "hello"
        /raw [ hello]
            " hello"
    """
    left = weechat.config_get_plugin("left_delimiter")
    right = weechat.config_get_plugin("right_delimiter")
    weechat.command(buffer, args.lstrip(left).rstrip(right))
    return weechat.WEECHAT_RC_OK


def my_modifier_cb(buf, modifier, modifier_data, string):
    if re.match(r'^\s.+$', string):
        # In the case where there actually is whitespace and we need to
        # deal with it, reset the input position to 0. We are only
        # dealing with the beginning of the string, so this has no
        # negative side affects. Without this, space causes the cursor
        # position to increment when attempting to type a whitespace
        # at the beginning of an otherwise-not-lead-with-whitespace string.
        weechat.buffer_set(modifier_data, "input_pos", "0")
    return string.lstrip()  # Remove _any_ whitespace at the start.


def get_left_delimiter():
    return weechat.config_get_plugin("left_delimiter")


def get_right_delimiter():
    return weechat.config_get_plugin("right_delimiter")


def get_usage():
    """ A dynamic function that produces usage based on the plugin's
    left_delimiter and right_delimiter options. """
    return get_left_delimiter() + " raw-text" + get_right_delimiter()


if __name__ == "__main__":
    for option, default_value in config.items():
        is_set = weechat.config_is_set_plugin(option)
        if not is_set:
            weechat.config_set_plugin(option, default_value)

    desc = f"""Send raw input (including whitespace in enclosed configured \
delimiters) to the current buffer.

Optional whitespace delimiters can be configured:

    plugins.var.python.{name}.left_delimiter
        - default: [
        - configured: {get_left_delimiter()}

    plugins.var.python.{name}.right_delimiter
        - default: ]
        - configured: {get_right_delimiter()}

Execute '/python reload unwanted_msg' to reflect configuration changes.
"""

    weechat.hook_modifier("input_text_content", "my_modifier_cb", str())
    weechat.hook_command("raw", desc, get_usage(), "", str(),
                         "raw_command_cb", str())



# -*- coding: utf-8 -*-
#
# Copyright (C) 2009 xt <xt@bash.no>
# Copyright (C) 2009-2012 Sebastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
#
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.7: make script compatible with Python 3.x,
#                  use string_match of WeeChat API
# 2010-06-03, Nils Görs <weechatter@arcor.de>:
#     version 0.6: option "toggle" added
# 2010-02-03, Alex Barrett <al.barrett@gmail.com>:
#     version 0.5: support wildcards in buffers list
# 2009-06-23, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.4: use modifier to show/hide nicklist on a buffer
# 2009-06-23, xt <xt@bash.no>:
#     version 0.3: use hiding/showing instead of disabling nicklist
# 2009-06-23, xt <xt@bash.no>:
#     version 0.2: use better check if buffer has nicklist
# 2009-06-22, xt <xt@bash.no>:
#     version 0.1: initial release

import weechat as w

SCRIPT_NAME    = "toggle_nicklist"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.7"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Auto show and hide nicklist depending on buffer name"

SCRIPT_COMMAND = "toggle_nicklist"

settings = {
    'action'  : 'hide', # show or hide nicklist in buffers list (next option)
    'buffers' : '',     # comma separated list
}

def display_action():
    w.prnt('', '%s: action = "%s"' % (SCRIPT_NAME, w.config_get_plugin("action")))

def display_buffers():
    w.prnt('', '%s: buffers = "%s"' % (SCRIPT_NAME, w.config_get_plugin("buffers")))

def get_buffers_list():
    buffers = w.config_get_plugin('buffers')
    if buffers == '':
        return []
    else:
        return buffers.split(',')

def nicklist_cmd_cb(data, buffer, args):
    ''' Command /nicklist '''
    if args == '':
        display_action()
        display_buffers()
    else:
        current_buffer_name = w.buffer_get_string(buffer, 'plugin') + '.' + w.buffer_get_string(buffer, 'name')
        if args == 'toggle':
            toggle = w.config_get_plugin('action')
            if toggle == 'show':
                w.config_set_plugin('action', 'hide')
                w.command('', '/window refresh')
            elif toggle == 'hide':
                w.config_set_plugin('action', 'show')
                w.command('', '/window refresh')
        if args == 'show':
            w.config_set_plugin('action', 'show')
            #display_action()
            w.command('', '/window refresh')
        elif args == 'hide':
            w.config_set_plugin('action', 'hide')
            #display_action()
            w.command('', '/window refresh')
        elif args == 'add':
            list = get_buffers_list()
            if current_buffer_name not in list:
                list.append(current_buffer_name)
                w.config_set_plugin('buffers', ','.join(list))
                #display_buffers()
                w.command('', '/window refresh')
            else:
                w.prnt('', '%s: buffer "%s" is already in list' % (SCRIPT_NAME, current_buffer_name))
        elif args == 'remove':
            list = get_buffers_list()
            if current_buffer_name in list:
                list.remove(current_buffer_name)
                w.config_set_plugin('buffers', ','.join(list))
                #display_buffers()
                w.command('', '/window refresh')
            else:
                w.prnt('', '%s: buffer "%s" is not in list' % (SCRIPT_NAME, current_buffer_name))

    return w.WEECHAT_RC_OK

def check_nicklist_cb(data, modifier, modifier_data, string):
    ''' The callback that checks if nicklist should be displayed '''

    buffer = w.window_get_pointer(modifier_data, "buffer")
    if buffer:
        buffer_name = w.buffer_get_string(buffer, 'plugin') + '.' + w.buffer_get_string(buffer, 'name')
        buffers_list = w.config_get_plugin('buffers')
        if w.config_get_plugin('action') == 'show':
            for buffer_mask in buffers_list.split(','):
                if w.string_match(buffer_name, buffer_mask, 1):
                    return "1"
            return "0"
        else:
            for buffer_mask in buffers_list.split(','):
                if w.string_match(buffer_name, buffer_mask, 1):
                    return "0"
            return "1"
    return "1"

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)

    w.hook_command(SCRIPT_COMMAND,
                   "Show or hide nicklist on some buffers",
                   "[show|hide|toggle|add|remove]",
                   "  show: show nicklist for buffers in list (hide nicklist for other buffers by default)\n"
                   "  hide: hide nicklist for buffers in list (show nicklist for other buffers by default)\n"
                   "toggle: show/hide nicklist for buffers in list\n"
                   "   add: add current buffer to list\n"
                   "remove: remove current buffer from list\n\n"
                   "Instead of using add/remove, you can set buffers list with: "
                   "/set plugins.var.python.%s.buffers \"xxx\". Buffers set in this "
                   "manner can start or end with * as wildcards to match multiple buffers."
                   % SCRIPT_NAME,
                   "show|hide|toggle|add|remove",
                   "nicklist_cmd_cb", "")
    w.hook_modifier('bar_condition_nicklist', 'check_nicklist_cb', '')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 - 2019  Stefan Wold <ratler@stderr.eu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# (This script requires WeeChat 0.4.2 or higher).
#
# WeeChat script that enables automatic OTP (OATH-TOTP) support for UnderNET's X
# and Login on Connect (LoC) authentication.
#
# The script will generate an OTP and automatically append it when it
# notices /msg x@channels.undernet.org login <username> <password> command.
# This allows OTP login when using irc.server.*.command to automatically
# sign in to the X service when connecting to an undernet server.
#
# Commands:
#  /uotp otp [server]
#  /uotp list
#  /uotp add <server> <seed>
#  /uotp remove <server>
#  /uotp enable <server>
#  /uotp disable <server>

from __future__ import print_function

SCRIPT_NAME    = "undernet_totp"
SCRIPT_AUTHOR  = "Stefan Wold <ratler@stderr.eu>"
SCRIPT_VERSION = "0.4.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Automatic OTP (OATH-TOTP) authentication with UnderNET's channel services (X) and Login on Connect (LoC)."
SCRIPT_COMMAND = "uotp"

HOOKS = {}

SETTINGS = {
    "otp_server_names": ("", "List of undernet server for which to enable OTP, use comma as separator"),
    "debug"           : ("off", "Debug output"),
}

import_ok = True

try:
    import weechat
except ImportError:
    print("This script must be run under WeeChat.")
    import_ok = False

try:
    import sys
    import hmac
    import re
    from base64 import b32decode
    from hashlib import sha1
    from struct import pack, unpack
    from time import time
    from binascii import unhexlify
except ImportError as err:
    print("Missing module(s) for %s: %s" % (SCRIPT_NAME, err))
    import_ok = False


def print_debug(message):
    if weechat.config_get_plugin('debug') == 'on':
        weechat.prnt("", "%s DEBUG: %s" % (SCRIPT_NAME, message))

def sprint(message, buffer=""):
    weechat.prnt(buffer, "%s: %s" % (SCRIPT_NAME, message))


def unhook(hook):
    global HOOKS

    if hook in HOOKS:
        print_debug('Unhooking %s' % hook)
        weechat.unhook(HOOKS[hook])
        del HOOKS[hook]


def unhook_all(server):
    for hook in [server+'.notice', server+'.modifier', server+'.modifier2']:
        unhook(hook)


def hook_all(server):
    print_debug("hook_all(%s)" % server)
    global HOOKS

    notice = server + '.notice'
    modifier = server + '.modifier'
    modifier2 = server + '.modifier2'

    if notice not in HOOKS:
        HOOKS[notice] = weechat.hook_signal("%s,irc_raw_in_notice" % server, "auth_success_cb", server)
    if modifier not in HOOKS:
        HOOKS[modifier] = weechat.hook_modifier("irc_out_privmsg", "totp_login_modifier_cb", server)
    if modifier2 not in HOOKS:
        HOOKS[modifier2] = weechat.hook_modifier("irc_out_pass", "totp_login_modifier_cb", server)


def totp_login_modifier_cb(data, modifier, server, cmd):
    if server == data and server in enabled_servers():
        if re.match(r'(?i)^(PRIVMSG x@channels.undernet.org :login .+ .+|PASS .*)', cmd):
            print_debug("totp_login_modifier_cb(%s)" % cmd)
            otp = generate_totp(server)
            if otp is not None:
                cmd += " %s" % otp
    return cmd


def auth_success_cb(server, signal, signal_data):
    if signal_data.startswith(":X!cservice@undernet.org NOTICE"):
        if re.match(r'^:X!cservice@undernet.org NOTICE .+ :AUTHENTICATION SUCCESSFUL', signal_data):
            unhook_all(server)

    return weechat.WEECHAT_RC_OK


def signal_cb(data, signal, server):
    if server in enabled_servers():
        print_debug('signal_cb(%s)' % signal)
        if signal == 'irc_server_connecting':
            hook_all(server)
        elif signal == 'irc_server_disconnected':
            unhook_all(server)

    return weechat.WEECHAT_RC_OK


def get_otp_cb(data, buffer, server):
    if server:
        server = [server]
    else:
        server = enabled_servers()

    for _server in server:
        otp = generate_totp(_server)
        if otp is not None:
            weechat.prnt("", "%s OTP: %s" % (_server, otp))

    return weechat.WEECHAT_RC_OK


def get_irc_servers():
    """ Returns a list of configured IRC servers in weechat"""
    serverptrlist = weechat.infolist_get('irc_server', '', '')
    serverlist = []
    while weechat.infolist_next(serverptrlist):
         serverlist.append(weechat.infolist_string(serverptrlist, 'name'))
    weechat.infolist_free(serverptrlist)
    return serverlist


def enabled_servers():
    """ Return a list of TOTP enabled servers. """
    serverlist = get_irc_servers()
    return [s for s in get_config_as_list('otp_server_names') if s in serverlist]


def disabled_servers():
    """ Return a list of configured TOTP servers that are currently disabled. """
    serverlist = get_irc_servers()
    server_seed_list = [server for server in serverlist
                        if weechat.string_eval_expression("${sec.data.%s_seed}" % server, {}, {}, {})
                        and server not in get_config_as_list('otp_server_names')]
    return [s for s in server_seed_list if s in serverlist]


def configured_servers():
    """ Return a lost of servers with an existing seed. """
    serverlist = get_irc_servers()
    return [s for s in serverlist if weechat.string_eval_expression("${sec.data.%s_seed}" % s, {}, {}, {})]


def generate_totp(server, period=30, buffer=""):
    print_debug('generate_totp(%s)' % server)
    seed = weechat.string_eval_expression("${sec.data.%s_seed}" % server, {}, {}, {})

    if not seed:
        sprint("No OATH-TOTP secret set, use: /uotp add %s <seed>" % server, buffer)
        return None

    if len(seed) == 40:  # Assume hex format
        seed = unhexlify(seed)
    else:
        seed = b32decode(seed.replace(" ", ""), True)

    def _ord(b):
        if sys.version_info[0] < 3 or type(b) == str:
            return ord(b)
        return b

    t = pack(">Q", int(time() / period))
    _hmac = hmac.new(seed, t, sha1).digest()
    o = _ord(_hmac[19]) & 15
    otp = (unpack(">I", _hmac[o:o+4])[0] & 0x7fffffff) % 1000000

    return '%06d' % otp


def config_update_cb(data, option, value):
    """ Reload hooks on configuration change. """
    print_debug("config_cb(%s)" % value)
    [hook_all(s.strip()) for s in value.split(',')]
    return weechat.WEECHAT_RC_OK


def options_cb(data, buffer, args):
    """ Script configuration callback """
    if not args:
        weechat.command("", "/help %s" % SCRIPT_COMMAND)
    args = args.strip().split(' ')
    opt = args[0]
    opt_args = args[1:]

    if opt == 'otp':
        if opt_args:
            servers = [opt_args[0]]
        else:
            servers = enabled_servers()
        for server in servers:
            otp = generate_totp(server, buffer=buffer)
            if otp:
                sprint("%s = %s" % (server, otp), buffer)
    elif opt == 'list':
        sprint("List of configured servers", buffer)
        for server in enabled_servers():
            weechat.prnt(buffer, "  - %s [enabled]" % server)
        for server in disabled_servers():
            weechat.prnt(buffer, "  - %s [disabled]" % server)
    elif opt == 'add':
        if len(opt_args) >= 2:
            if opt_args[0] not in enabled_servers() and opt_args[0] in get_irc_servers():
                #weechat.command("", "/secure set %s_seed %s" % (opt_args[0], opt_args[1]))
                try:
                    add_server(opt_args[0], opt_args[1:])
                    sprint("server '%s' was successfully added" % opt_args[0], buffer)
                except Exception as ex:
                    sprint("invalid TOTP seed provided", buffer)
            elif opt_args[0] not in get_irc_servers():
                sprint("No server named '%s' was found, see /help server" % opt_args[0], buffer)
            else:
                sprint("OTP already configured for '%s', to change <seed> remove the existing one first." % opt_args[0], buffer)
        else:
            sprint("/uotp -- invalid argument, valid command is /uotp add <server> <seed>", buffer)
    elif opt == 'remove':
        if opt_args[0] in enabled_servers() or opt_args[0] in disabled_servers():
            remove_server(opt_args[0], True)
            sprint("server '%s' was successfully removed" % opt_args[0], buffer)
        else:
            sprint("failed to remove server, '%s' not found" % opt_args[0], buffer)
    elif opt == 'enable':
        if opt_args and opt_args[0] not in enabled_servers():
            if opt_args[0] in get_irc_servers():
                add_server(opt_args[0])
                sprint("server '%s' was successfully enabled" % opt_args[0], buffer)
            else:
                sprint("No server named '%s' was found, see /help server" % opt_args[0], buffer)
        else:
            sprint("OTP is already enabled for the server '%s'." % opt_args[0], buffer)
    elif opt == 'disable':
        if opt_args and opt_args[0] in enabled_servers():
            remove_server(opt_args[0])
        else:
            sprint("OTP does not seem to be enabled for '%s'" % opt_args[0], buffer)
    elif opt:
        sprint("/uotp: invalid option -- '%s'" % opt, buffer)
        weechat.command("", "/help %s" % SCRIPT_COMMAND)

    return weechat.WEECHAT_RC_OK


def get_config_as_list(option):
    """ Return comma-separated <option> as a list. """
    return [o.strip() for o in weechat.config_get_plugin(option).strip().split(',')]


def add_server(server, seed=None):
    """ Append new server to the plugin configuration. """
    if seed:  # Test seed
        if len(seed[0]) == 40:
            unhexlify(seed[0])
            seed = seed[0]
        else:
            b32decode(''.join(seed).replace(" ", ""), True)
            seed = ' '.join(seed)
        weechat.command("", "/secure set %s_seed %s" % (server, seed))

    servers = get_config_as_list("otp_server_names")
    if server not in servers:
        servers.append(server)
        weechat.config_set_plugin("otp_server_names", ','.join(servers))


def remove_server(server, remove_seed=False):
    """ Remove server from the plugin configuration. """
    if remove_seed and weechat.string_eval_expression("${sec.data.%s_seed}" % server, {}, {}, {}):
        weechat.command("", "/secure del %s_seed" % server)

    servers = get_config_as_list("otp_server_names")
    if server in servers:
        servers = [v for v in servers if v != server]
        weechat.config_set_plugin("otp_server_names", ','.join(servers))


def hide_secret_cb(data, modifier, modifier_data, cmd):
    """ Callback to hide seed secret during input by replacing the seed with asterisks (*). """
    match = re.search(r'(?i)^/uotp add \w+ (.+)', cmd)
    if match:
        last_arg = match.group(1)
        rep = '*' * len(last_arg)
        cmd = re.sub('%s$' % last_arg, rep, cmd)
    return cmd


def server_completion_cb(data, completion_item, buffer, completion):
    """ Enabled or disabled server completion callback. """
    print_debug("completion " + ', '.join(globals()[data]()))
    for server in globals()[data]():
        weechat.hook_completion_list_add(completion, server, 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK


if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                        SCRIPT_DESC, "", ""):
        version = weechat.info_get("version_number", "") or 0
        if int(version) < 0x00040200:
            weechat.prnt("", "%s requires WeeChat >= 0.4.2 for secure_data support." % SCRIPT_NAME)
            weechat.command("", "/wait 1ms /python unload %s" % SCRIPT_NAME)

        weechat.hook_command(SCRIPT_COMMAND,
                             "Generate a One-Time Password (TOTP) for service authentication or login on connect.",
                             "otp [server] || list || add <server> <seed> || remove <server> || enable <server> || disable <server>",
                             "    otp: generate one-time password for one or all servers\n"+
                             "   list: list configured servers\n"+
                             "    add: add one-time password (TOTP) seed for an existing irc server\n"+
                             " remove: delete one-time password configuration for a server\n"+
                             " enable: re-enable one-time password authentication for a server\n"+
                             "disable: disable one-time password authentiction without removing the seed for a server\n\n"+
                             "Examples:\n"+
                             "  /uotp otp\n"+
                             "  /uotp otp freenode\n"+
                             "  /uotp list\n"+
                             "  /uotp add freenode 4c6fdb7d0659bae2a16d23bab99678462b9f7897\n"+
                             "  /uotp add freenode jrx5 w7ig lg5o filn eo5l tfty iyvz 66ex\n"+
                             "  /uotp remove freenode\n"+
                             "  /uotp enable freenode\n"+
                             "  /uotp disable freenode",
                             "otp %(irc_servers)"
                             " || list"
                             " || add %(irc_servers)"
                             " || remove %(configured_servers)"
                             " || enable %(irc_servers)"
                             " || disable %(disabled_servers)",
                             "options_cb", "")
        weechat.hook_signal("irc_server_connecting", "signal_cb", "")
        weechat.hook_signal("irc_server_disconnected", "signal_cb", "")
        weechat.hook_config("plugins.var.python.undernet_totp.otp_server_names", "config_update_cb", "")
        weechat.hook_completion("configured_servers", "list of otp configured servers", "server_completion_cb", "configured_servers")
        weechat.hook_completion("disabled_servers", "list of disabled servers", "server_completion_cb", "enabled_servers")
        weechat.hook_modifier("input_text_display", "hide_secret_cb", "")

        for option, default_value in SETTINGS.items():
            if weechat.config_get_plugin(option) == "":
                weechat.config_set_plugin(option, default_value[0])
            weechat.config_set_desc_plugin(option, '%s (default: %s)' % (default_value[1], default_value[0]))

        # For now we enable the hooks until it's possible to force script plugins to
        # load before the irc plugin on weechat startup, otherwise the irc_server_connecting signal
        # get missed.
        for server in enabled_servers():
            hook_all(server)



"""
    mop.py

    author: Adam Saponara <saponara AT gmail DOT com>
      desc: Op everyone in the current channel
     usage: /mop
   license: GPLv3

   history:
       0.1 - 2016-02-23, initial script
"""

import weechat

weechat.register("mop", "Adam Saponara", "0.1", "GPL3", "Op everyone in the current channel", "", "")

def mop_cmd(data, buffer, args):
    nicks = []
    nicklist = weechat.infolist_get("nicklist", buffer, "")
    while weechat.infolist_next(nicklist):
        nick = weechat.infolist_string(nicklist, "name")
        nick_type = weechat.infolist_string(nicklist, "type")
        nick_prefix = weechat.infolist_string(nicklist, "prefix")
        if nick_type == "nick" and nick_prefix != "@":
            nicks.append(nick)
    weechat.infolist_free(nicklist)
    if len(nicks) > 0:
        weechat.command(buffer, "/op " + " ".join(nicks))
    return weechat.WEECHAT_RC_OK

hook = weechat.hook_command("mop", "Op everyone in the current channel", "", "", "", "mop_cmd", "")



"""Unlike most automatic op/voice plugins, this plugin uses the reop (R) and invite (I) lists of a channel for automatic op and voice control. This allows for transparent and centralised channel administration while responsibility is shared with all channel operators. This is particularly useful for networks that have no services (e.g., IRCnet).


Subcommands

    list: Show the reop and invite lists of all activated channels.
    reload: Reload the reop and invite lists for all activated channels.


Configuration

The plugin can be enabled or disabled via the following setting.

    /set plugins.var.python.reop.enabled

The plugin can be activated for channels in a particular network by adding a key-value pair to the configuration with the name of the network as the key and a comma separated list of channel names as its value. E.g., the following activates the plugin for channels #chan1 and #chan2 on ircnet.

    /set plugins.var.python.reop.net.ircnet #chan1,#chan2

Servers or other clients may also set user privileges. To avoid unnecessary chatter, the current user privileges are checked some time after a user joins. Only if the privilege level is lower than configured in the reop and invite lists, the appropriate privileges are granted. The delay is configured via the following setting.

    /set plugins.var.python.reop.delay 1000


Usage

Operator control is managed by manipulating the reop (R) list.

Grant operator privileges.

    /mode #chan1 +R nick!user@host.name

Revoke operator privileges.

    /mode #chan1 -R nick!user@host.name

Voice control is managed by manipulating the invite (I) list.

Grant voice privileges.

    /mode #chan1 +I nick!user@host.name

Revoke voice privileges.

    /mode #chan1 -I nick!user@host.name
"""
from weechat import (
    WEECHAT_RC_OK, buffer_search, command, config_get_plugin,
    config_set_plugin, hook_command, hook_modifier, hook_signal, hook_timer,
    info_get, info_get_hashtable, infolist_get, infolist_free, infolist_next,
    infolist_string, prnt, register, string_match)

_name = 'reop'
_author = 'Jeroen F.J. Laros <jlaros@fixedpoint.nl>'
_version = '1.0.0'
_license = 'MIT'
_description = 'Use reop and invite lists for automatic op/voice control.'

reop_data = {}


def _parse(signal, signal_data):
    """Parse a signal."""
    network = signal.split(',')[0]
    msg = info_get_hashtable('irc_message_parse', {'message': signal_data})
    buf = info_get('irc_buffer', '{},{}'.format(network, msg['channel']))

    return network, msg, buf


def _level(network, channel, nick):
    """Determine the current privilege level."""
    nicks = infolist_get(
        'irc_nick', '', '{},{},{}'.format(network, channel, nick))

    prefix = ' '
    while infolist_next(nicks):
        prefix = infolist_string(nicks, 'prefix')

    infolist_free(nicks)

    # y q a o h v *
    return [' ', '+', '@'].index(prefix)


def timer_cb(data, remaining_calls):
    """Timer callback."""
    buf, level, network, channel, nick = reop_data['cmd'].pop()

    if _level(network, channel, nick) < level:
        if level == 2:
            command(buf, '/mode {} +o {}'.format(channel, nick))
        elif level == 1:
            command(buf, '/mode {} +v {}'.format(channel, nick))

    return WEECHAT_RC_OK


def _schedule(buf, level, network, channel, nick):
    """Schedule a mode change."""
    reop_data['cmd'].append((buf, level, network, channel, nick))
    hook_timer(reop_data['delay'], 0, 1, 'timer_cb', '')


def _join(buf, channel):
    """Get reop and invite lists."""
    reop_data['hide'] += 2

    command(buf, '/mode {} R'.format(channel))
    command(buf, '/mode {} I'.format(channel))


def join_cb(data, signal, signal_data):
    """Join callback."""
    network, msg, buf = _parse(signal, signal_data)

    cache = reop_data['cache']
    if network in cache:
        channel = msg['channel']

        if channel in cache[network]:
            nick = msg['nick']
            host = msg['host']

            if nick != info_get('irc_nick', network):
                # Someone is joining.
                for mask in cache[network][channel]['reop']:
                    if string_match(host, mask, 1):
                        _schedule(buf, 2, network, channel, nick)
                        return WEECHAT_RC_OK

                for mask in cache[network][channel]['invite']:
                    if string_match(host, mask, 1):
                        _schedule(buf, 1, network, channel, nick)
                        return WEECHAT_RC_OK
            else:
                # User is joining.
                cache[network][channel] = {'reop': set(), 'invite': set()}
                _join(buf, channel)

    return WEECHAT_RC_OK


def list_cb(data, signal, signal_data):
    """List callback."""
    network, msg, _ = _parse(signal, signal_data)

    cache = reop_data['cache']
    if network in cache:
        channel = msg['channel']

        if channel in cache[network]:
            mask = msg['text']
            command = msg['command']

            if command == '344':
                cache[network][channel]['reop'].add(mask)
            elif command == '346':
                cache[network][channel]['invite'].add(mask)

    return WEECHAT_RC_OK


def mode_cb(data, signal, signal_data):
    """Mode callback."""
    network, msg, _ = _parse(signal, signal_data)

    cache = reop_data['cache']
    if network in cache:
        channel = msg['channel']

        if channel in cache[network]:
            mode, mask = msg['text'].split(' ')[:2]

            if mode == '+R':
                cache[network][channel]['reop'].add(mask)
            elif mode == '-R':
                cache[network][channel]['reop'].discard(mask)
            elif mode == '+I':
                cache[network][channel]['invite'].add(mask)
            elif mode == '-I':
                cache[network][channel]['invite'].discard(mask)

    return WEECHAT_RC_OK


def _format(d, indent=0):
    """Pretty print a dictionary."""
    for key in d:
        prnt('', '{}{}'.format(indent * ' ', key))

        if isinstance(d, dict):
            _format(d[key], indent + 2)


def _init_cache():
    """Initialise the cache."""
    reop_data.update({
        'cache': {}, 'cmd': [], 'hide': 0,
        'delay': int(config_get_plugin('delay'))})

    networks = infolist_get('irc_server', '', '')

    while infolist_next(networks):
        network = infolist_string(networks, 'name')
        channels = config_get_plugin('net.{}'.format(network))

        if channels:
            cache = reop_data['cache']
            cache[network] = {}

            for channel in channels.split(','):
                cache[network][channel] = {
                    'reop': set(), 'invite': set()}

                buf = buffer_search('irc', '{}.{}'.format(network, channel))
                if buf:
                    _join(buf, channel)

    infolist_free(networks)


def command_cb(data, buf, args):
    """Command callback."""
    cmd = args.split(' ')[0]

    if cmd == 'list':
        prnt('', '\nReop cache (pending {}):\n'.format(reop_data['hide']))
        _format(reop_data['cache'])
    elif cmd == 'reload':
        _init_cache()

    return WEECHAT_RC_OK


def modifier_cb(data, modifier, modifier_data, string):
    """Print modifier callback."""
    tagset = set(modifier_data.split(';')[-1].split(','))

    if reop_data['hide']:
        if {'irc_344', 'irc_346'} & tagset:
            return ''
        if {'irc_345', 'irc_347'} & tagset:
            reop_data['hide'] -= 1
            return ''

    return string


def _config():
    """Configuration."""
    if not config_get_plugin('enabled'):
        config_set_plugin('enabled', 'on')
    status = config_get_plugin('enabled')

    if not config_get_plugin('delay'):
        config_set_plugin('delay', '5000')

    return status == 'on'


def main():
    """Initialisation."""
    register(_name, _author, _version, _license, _description, '', '')

    if not _config():
        return

    _init_cache()

    hook_signal('*,irc_in2_join', 'join_cb', '')
    hook_signal('*,irc_in2_344', 'list_cb', '')
    hook_signal('*,irc_in2_346', 'list_cb', '')
    hook_signal('*,irc_in2_mode', 'mode_cb', '')

    hook_modifier('weechat_print', 'modifier_cb', '')

    hook_command(
        'reop', '{}\n\n{}'.format(_description, __doc__), 'list | reload',
        '', '', 'command_cb', '')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2019 by nils_2 <weechatter@arcor.de>
#
# for easy toggling current buffer logging.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2019-06-24: nils_2, (freenode.#weechat)
#       0.1 : initial version
#
# requires: WeeChat version 1.0
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat,re

except Exception:
    print('This script must be run under WeeChat.')
    print('Get WeeChat now at: https://www.weechat.org/')
    quit()

SCRIPT_NAME     = 'log'
SCRIPT_AUTHOR   = 'nils_2 <weechatter@arcor.de>'
SCRIPT_VERSION  = '0.1'
SCRIPT_LICENSE  = 'GPL'
SCRIPT_DESC     = 'for easy toggling current buffer logging'

# eval_expression():  to match ${color:nn} tags
regex_color=re.compile('\$\{color:([^\{\}]+)\}')

# ==========================[ eval_expression() ]========================
def substitute_colors(text,window):
    global version
    if int(version) >= 0x00040200:
        buffer_ptr = weechat.window_get_pointer(window,"buffer")
        return weechat.string_eval_expression(text, {"window": window, "buffer": buffer_ptr}, {}, {})
    # substitute colors in output
    return re.sub(regex_color, lambda match: weechat.color(match.group(1)), text)

# ============================[ subroutines ]============================
def log_cmd_cb(data, buffer, args):
    argv = args.strip().split(' ')

    log_level = infolist_log_buffer(buffer)

    if args == "" or (argv[0].lower() == 'show'):
        # no args given. display log level of current buffer
        weechat.prnt(buffer,'log level: %s' % log_level)
        return weechat.WEECHAT_RC_OK

    if (argv[0].lower() == 'enable') or (argv[0].lower() == 'on'):
        if log_level != 'disabled':
            return weechat.WEECHAT_RC_OK    # buffer already logging!
        else:
            enable_check(log_level,buffer)
        return weechat.WEECHAT_RC_OK

    if (argv[0].lower() == 'disable') or (argv[0].lower() == 'off'):
        if log_level == 'disabled':
            return weechat.WEECHAT_RC_OK    # buffer already disabled!
        else:
            disable_check(log_level,buffer)
        return weechat.WEECHAT_RC_OK

    if (argv[0].lower() == 'toggle'):
        if log_level == 'disabled':
            enable_check(log_level,buffer)
        else:
            disable_check(log_level,buffer)
        return weechat.WEECHAT_RC_OK

    return weechat.WEECHAT_RC_OK

# ===============================[ logger() ]=============================
def enable_check(log_level,buffer):
    log_level = buffer_get_string_log_level(buffer)
    if log_level:
        if not str(log_level).isnumeric() or (int(log_level) < 0) or (int(log_level) > 9):
            log_level = 9 # invalid log level, set default
        weechat.command(buffer,'/logger set %s' % log_level)
        buffer_del_log_level(buffer)
    else:   # no logging and no localvar.
        weechat.command(buffer,'/logger set 9')
    return weechat.WEECHAT_RC_OK

def disable_check(log_level,buffer):
    buffer_set_string_log_level(buffer,log_level)   # store old log level in localvar!
    weechat.command(buffer,'/logger disable')
    return weechat.WEECHAT_RC_OK

# =============================[ localvars() ]============================
def buffer_get_string_log_level(buffer):
    return weechat.buffer_get_string(buffer,'localvar_log_level')

def buffer_set_string_log_level(buffer,log_level):
    weechat.buffer_set(buffer, 'localvar_set_log_level', '%s' % log_level)
    return weechat.WEECHAT_RC_OK

def buffer_del_log_level(buffer):
    weechat.command(buffer,'/buffer set localvar_del_log_level')
    return weechat.WEECHAT_RC_OK

# =============================[ infolist() ]============================
def infolist_log_buffer(ptr_buffer):
    log_level = None
    infolist = weechat.infolist_get('logger_buffer','','')
    while weechat.infolist_next(infolist):
        bpointer = weechat.infolist_pointer(infolist, 'buffer')
        if ptr_buffer == bpointer:
            log_enabled = weechat.infolist_integer(infolist, 'log_enabled')
            log_level = weechat.infolist_integer(infolist, 'log_level')

    weechat.infolist_free(infolist)                  # free infolist()
    if not log_level:
        return 'disabled'
    else:
        return log_level
# ================================[ main ]===============================
if __name__ == '__main__':
    global version
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get('version_number', '') or 0

        # get weechat version (1.0) and store it
        if int(version) >= 0x01000000:

            weechat.hook_command(SCRIPT_NAME,SCRIPT_DESC,
                             'enable|on||'
                             'disable|off||'
                             'toggle||'
                             'show',
                             '  enable/on: enable logging on current buffer, with default log-level (note: log-level from localvar will be used, if possible)\n'
                             'disable/off: disable logging on current buffer (note: log-level is stored in localvar)\n'
                             '     toggle: will toggle logging on current buffer\n'
                             '       show: will print current log-level to buffer (default)\n'
                             '\n'
                             'Examples:\n'
                             '  /log toggle',
                             'enable||'
                             'disable||'
                             'on||'
                             'off||'
                             'toggle||'
                             'show',
                             'log_cmd_cb', '')

        else:
            weechat.prnt('','%s%s %s' % (weechat.prefix('error'),SCRIPT_NAME,': needs version 1.0 or higher'))
            weechat.command('','/wait 1ms /python unload %s' % SCRIPT_NAME)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012-2018 by nils_2 <weechatter@arcor.de>
#
# save and restore global and/or buffer command history
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2021-05-02, Sébastien Helleu <flashcode@flashtux.org>
#       0.8.2 : add compatibility with WeeChat >= 3.2 (XDG directories)
#
# 2018-08-04: nils_2 (freenode.#weechat)
#       0.8 : add option 'save_buffer'
#           : thanks catbeard for revise the help text
#
# 2018-08-01: nils_2 (freenode.#weechat)
#       0.7 : rmodifier routine removed
#           : bug fixes
#
# 2017-12-14: Sébastien Helleu <flashcode@flashtux.org>
#       0.6 : rename command "/autosetbuffer" by "/buffer_autoset" in example
#
# 2015-04-05: nils_2 (freenode.#weechat)
#       0.5 : change priority of hook_signal('buffer_opened') to 100
#
# 2013-01-25: nils_2 (freenode.#weechat)
#       0.4 : make script compatible with Python 3.x
#
# 2013-01-20: nils_2, (freenode.#weechat)
#       0.3: fix wrong command argument in help-text
#
# 2012-12-21: nils_2, (freenode.#weechat)
#       0.2 : fix UnicodeEncodeError
#
# 2012-12-09: nils_2, (freenode.#weechat)
#       0.1 : initial release
#
# thanks to nestib for help with regex and for the rmodifier idea
#
# requires: WeeChat version 0.4.0
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat,re,os

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = 'histman'
SCRIPT_AUTHOR   = 'nils_2 <weechatter@arcor.de>'
SCRIPT_VERSION  = '0.8.2'
SCRIPT_LICENSE  = 'GPL'
SCRIPT_DESC     = 'save and restore global and/or buffer command history'

OPTIONS         = { 'number'       : ('0','number of history commands/text to save. A positive number will save from oldest to latest, a negative number will save from latest to oldest. 0 = save whole history (e.g. -10 will save the last 10 history entries'),
                    'pattern'      : ('(.*password|.*nickserv|/quit)','a simple regex to ignore commands/text. Empty value disable pattern matching'),
                    'skip_double'  : ('on','skip lines that already exists (case sensitive)'),
                    'save'         : ('all','define what should be saved from history. Possible values are \"command\", \"text\", \"all\". This is a fallback option (see /help ' + SCRIPT_NAME +')'),
                    'history_dir'  : ('%h/history','locale cache directory for history files (\"%h\" will be replaced by WeeChat data directory)'),
                    'save_global'  : ('off','save global history, possible values are \"command\", \"text\", \"all\" or \"off\"(default: off)'),
                    'save_buffer'  : ('off','save buffer history from all buffers, possible values are \"on\", \"off\". Using this option, localvar from buffer will be ignored (default: off)'),
                    'min_length'   : ('2','minimum length of command/text (default: 2)'),
                    'buffer_close' : ('off','save command history, when buffer will be closed (default: off)'),
                  }

filename_global_history = 'global_history'
possible_save_options = ['command', 'text', 'all']

history_list = []

# =================================[ save/restore buffer history ]================================
def save_history():
    global history_list
    # get buffers
    ptr_infolist_buffer = weechat.infolist_get('buffer','','')

    while weechat.infolist_next(ptr_infolist_buffer):
        ptr_buffer = weechat.infolist_pointer(ptr_infolist_buffer,'pointer')

        # check for localvar_save_history
        if not weechat.buffer_get_string(ptr_buffer, 'localvar_save_history') and OPTIONS['save_buffer'].lower() == 'off':
            continue

        plugin = weechat.buffer_get_string(ptr_buffer, 'localvar_plugin')
        name = weechat.buffer_get_string(ptr_buffer, 'localvar_name')
        filename = get_filename_with_path('%s.%s' % (plugin,name))

        get_buffer_history(ptr_buffer)
        if len(history_list):
            write_history(filename)

    weechat.infolist_free(ptr_infolist_buffer)

    if OPTIONS['save_global'].lower() != 'off':
        get_buffer_history('')  # buffer pointer (if not set, return global history)
        if len(history_list):
            write_history(filename_global_history)

def get_buffer_history(ptr_buffer):
    global history_list

    history_list = []
    ptr_buffer_history = weechat.infolist_get('history',ptr_buffer,'')

    if not ptr_buffer_history:
        return

    while weechat.infolist_next(ptr_buffer_history):
        line = weechat.infolist_string(ptr_buffer_history, 'text')

        if add_buffer_line(line,ptr_buffer):
            history_list.insert(0,line)

    weechat.infolist_free(ptr_buffer_history)

# return 1; line will be saved
# return 0; line won't be saved
def add_buffer_line(line, ptr_buffer):
    global history_list

    # min_length reached?
    if len(line) < int(OPTIONS['min_length']):
        return 0

    add_line = 0

    # ptr_buffer empty = global history
    if ptr_buffer: # buffer history
        save_history = weechat.buffer_get_string(ptr_buffer, 'localvar_save_history').lower()
        if not save_history.lower() in possible_save_options:
            save_history = OPTIONS['save'].lower()
    else:       # global history
        save_history = OPTIONS['save_global'].lower()
        if not OPTIONS['save_global'].lower() in possible_save_options:
            save_history = OPTIONS['save'].lower()

    # no save option given? save nothing
    if save_history == '':
        return 0

    if save_history == 'command':
        command_chars = ("/%s" % weechat.config_string(weechat.config_get('weechat.look.command_chars')))
        # a valid command must have at least two chars and first and second char are not equal!
        if len(line) > 1 and line[0] in command_chars and line[0] != line[1]:
            add_line = 1
        else:
            return 0
    elif save_history == 'text':
        command_chars = ("/%s" % weechat.config_string(weechat.config_get('weechat.look.command_chars')))
        # test for "//" = text
        if line[0] == line[1] and line[0] in command_chars:
            add_line = 1
        # test for "/n" = command
        elif line[0] != line[1] and line[0] in command_chars:
            return 0
        else:
            add_line = 1
    elif save_history == 'all':
        add_line = 1
    else: # not one of given values. save nothing!
        return 0

    # lines already exist?
    if OPTIONS['skip_double'].lower() == 'on':
        history_list2 = []
        history_list2 = [element.lower() for element in history_list]
        if line.lower() in history_list2:
            return 0
        else:
            add_line = 1
    else:
        add_line = 1

    if add_line == 0:
        return 0

    pattern_matching = 0
    # pattern matching for user option
    if OPTIONS['pattern'] != '':
        filter_re=re.compile(OPTIONS['pattern'], re.I)
        # pattern matched
        if filter_re.match(line):
            pattern_matching = 1

    if add_line == 1 and pattern_matching == 0:
        return 1
    return 0

# =================================[ read/write history to file ]=================================
def read_history(filename,ptr_buffer):
    global_history = 0

    # get buffer_autoset_option as a fallback. could happen that the localvar is not already set to buffer
    plugin = weechat.buffer_get_string(ptr_buffer, 'localvar_plugin')
    name = weechat.buffer_get_string(ptr_buffer, 'localvar_name')
    buffer_autoset_option = ('buffer_autoset.buffer.%s.%s.localvar_set_save_history' % (plugin,name))
    buffer_autoset_option = weechat.config_get(buffer_autoset_option)

    # global history does not use buffer pointers!
    if filename == filename_global_history:
        global_history = 1

    filename = get_filename_with_path(filename)

    # filename exists?
    if not os.path.isfile(filename):
        return

    # check for buffer history (0, global = 1)
    if global_history == 0 and OPTIONS['save_buffer'].lower() == 'off':
        # localvar_save_history exists for buffer?
        if not ptr_buffer and not buffer_autoset_option and not weechat.buffer_get_string(ptr_buffer, 'localvar_save_history'):
            return
    hdata = weechat.hdata_get('history')
    if not hdata:
        return

    try:
        f = open(filename, 'r')
#        for line in f.xreadlines():    # old python 2.x
        for line in f:                  # should also work with python 2.x
#            line = line.decode('utf-8')
            line = str(line.strip())
            if ptr_buffer:
                # add to buffer history
                weechat.hdata_update(hdata, '', { 'buffer': ptr_buffer, 'text': line })
            else:
                # add to global history
                weechat.hdata_update(hdata, '', { 'text': line })
        f.close()
    except:
        if global_history == 1:
            weechat.prnt('','%s%s: Error loading global history from "%s"' % (weechat.prefix('error'), SCRIPT_NAME, filename))
        else:
            name = weechat.buffer_get_string(ptr_buffer, 'localvar_name')
            weechat.prnt('','%s%s: Error loading buffer history for buffer "%s" from "%s"' % (weechat.prefix('error'), SCRIPT_NAME, name, filename))
        raise

def write_history(filename):
    global history_list

    filename = get_filename_with_path(filename)

    if OPTIONS['number'] != '' and OPTIONS['number'].isdigit():
        if int(OPTIONS['number']) < 0:
            save_from_position = len(history_list) - abs(int(OPTIONS['number']))
            if save_from_position == len(history_list) or save_from_position < 0:
                save_from_position = 0
        elif int(OPTIONS['number']) > 0:
            save_to_position = int(OPTIONS['number'])
            if save_to_position > len(history_list):
                save_to_position = len(history_list)
        else:
            save_from_position = 0
    try:
        f = open(filename, 'w')

        if int(OPTIONS['number']) <= 0:
            i = save_from_position
            # for i in range(len(a)):
            while i < len(history_list):
                f.write('%s\n' % history_list[i])
                i = i + 1
        if int(OPTIONS['number']) > 0:
            i = 0
            while i < save_to_position:
                f.write('%s\n' % history_list[i])
                i = i + 1
        f.close()

    except:
        weechat.prnt('','%s%s: Error writing history to "%s"' % (weechat.prefix('error'),SCRIPT_NAME,filename))
        raise

def get_filename_with_path(filename):
    options = {
        'directory': 'data',
    }
    path = weechat.string_eval_path_home(OPTIONS['history_dir'], {}, {}, options)
    return os.path.join(path, filename)

def config_create_dir():
    options = {
        'directory': 'data',
    }
    dir = weechat.string_eval_path_home(OPTIONS['history_dir'], {}, {}, options)
    if not os.path.isdir(dir):
        os.makedirs(dir, mode=0o700)

# ===========================================[ Hooks() ]==========================================
def create_hooks():
    # create hooks
    weechat.hook_signal('quit', 'quit_signal_cb', '')
    weechat.hook_signal('upgrade_ended', 'upgrade_ended_cb', '')
    # low priority for hook_signal('buffer_opened') to ensure that buffer_autoset hook_signal() runs first
    weechat.hook_signal('1000|buffer_opened', 'buffer_opened_cb', '')
    weechat.hook_config('plugins.var.python.' + SCRIPT_NAME + '.*', 'toggle_refresh', '' )
    weechat.hook_signal('buffer_closing', 'buffer_closing_cb', '')

def quit_signal_cb(data, signal, signal_data):
    # create dir, if not exist
    config_create_dir()
    save_history()
    return weechat.WEECHAT_RC_OK

def buffer_opened_cb(data, signal, signal_data):
    ptr_buffer = signal_data
    plugin = weechat.buffer_get_string(ptr_buffer, 'localvar_plugin')
    name = weechat.buffer_get_string(ptr_buffer, 'localvar_name')
    filename = get_filename_with_path('%s.%s' % (plugin,name))
    read_history(filename,ptr_buffer)
    return weechat.WEECHAT_RC_OK

def buffer_closing_cb(data, signal, signal_data):
    if OPTIONS['buffer_close'].lower() == 'on' and signal_data:
        # check for localvar_save_history
        if not weechat.buffer_get_string(signal_data, 'localvar_save_history') and OPTIONS['save_buffer'].lower() == 'off':
            return weechat.WEECHAT_RC_OK

        plugin = weechat.buffer_get_string(signal_data, 'localvar_plugin')
        name = weechat.buffer_get_string(signal_data, 'localvar_name')
        filename = get_filename_with_path('%s.%s' % (plugin,name))
        get_buffer_history(signal_data)

        if len(history_list):
            write_history(filename)
    return weechat.WEECHAT_RC_OK

def upgrade_ended_cb(data, signal, signal_data):
    weechat.buffer_set(weechat.buffer_search_main(), 'localvar_set_histman', 'on')
    return weechat.WEECHAT_RC_OK

def histman_cmd_cb(data, buffer, args):
    if args == '':
        weechat.command('', '/help %s' % SCRIPT_NAME)
        return weechat.WEECHAT_RC_OK

    argv = args.strip().split(' ', 1)
    if len(argv) == 0:
        return weechat.WEECHAT_RC_OK

    if argv[0].lower() == 'save':
        config_create_dir()
        save_history()
    elif argv[0].lower() == 'list':
        weechat.command('','/set *.localvar_set_save_history')
    else:
        weechat.command('', '/help %s' % SCRIPT_NAME)

    return weechat.WEECHAT_RC_OK

# ================================[ weechat options & description ]===============================
def init_options():
    for option,value in list(OPTIONS.items()):
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, value[0])
            weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = weechat.config_get_plugin(option)

def toggle_refresh(pointer, name, value):
    global OPTIONS
    option = name[len('plugins.var.python.' + SCRIPT_NAME + '.'):]        # get optionname
    OPTIONS[option] = value                                            # save new value
    weechat.bar_item_update(SCRIPT_NAME)
    return weechat.WEECHAT_RC_OK

# ================================[ main ]===============================
if __name__ == '__main__':
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get('version_number', '') or 0

        weechat.hook_command(SCRIPT_NAME, SCRIPT_DESC, '[save] || [list]',
                            '  save: force to save command history\n'
                            '  list: list local buffer variable(s)\n'
                            '\n'
                            'If you \"/quit\" WeeChat, the script will automatically save the command history to a file.\n'
                            'You can also force the script to save command history, when you close a buffer with /buffer close.\n'
                            'If you restart WeeChat, the command history will be restored, when buffer opens again.\n'
                            'To save and restore \"global\" command history, use option \"save_global\".\n'
                            '\n'
                            'The command history of a buffer will be saved \"only\", if the the local variable \"save_history\" is set.\n'
                            'You will need script \"buffer_autoset.py\" to make a local variabe persistent (see examples, below)!!\n'
                            '\n'
                            'You can use option \"save_buffer\" to save command history for *all* buffers, localvar will be ignored using\n'
                            'this option.\n'
                            'You can use following values for local variable:\n'
                            '  command: save commands only\n'
                            '     text: save text only (text sent to a channel buffer)\n'
                            '      all: save commands and text\n'
                            '\n'
                            'Examples:\n'
                            ' save the command history manually (for example with /cron script or with an /trigger):\n'
                            '   /' + SCRIPT_NAME + ' save\n'
                            ' save and restore command history for buffer #weechat on freenode (text only):\n'
                            '   /buffer_autoset add irc.freenode.#weechat localvar_set_save_history text\n'
                            ' save and restore command history for weechat core buffer (commands only):\n'
                            '   /buffer_autoset add core.weechat localvar_set_save_history command\n',
                            'save %-'
                            '|| list %-',
                            'histman_cmd_cb', '')

        if int(version) >= 0x00040000:
            if weechat.buffer_get_string(weechat.buffer_search_main(),'localvar_histman') == 'on':
                init_options()
                create_hooks()
                weechat.prnt('','%s%s: do not start this script two times. command history was already restored, during this session!' % (weechat.prefix('error'),SCRIPT_NAME))
            else:
                init_options()
                # create dir, if not exist
                config_create_dir()

                # look for global_history
                if OPTIONS['save_global'].lower() != 'off':
                    read_history(filename_global_history,'')

                # core buffer is already open on script startup. Check manually!
                filename = get_filename_with_path('core.weechat')
                read_history('core.weechat',weechat.buffer_search_main())

                create_hooks()

                # set localvar, to start script only once!
                weechat.buffer_set(weechat.buffer_search_main(), 'localvar_set_histman', 'on')
        else:
            weechat.prnt('','%s%s: needs version 0.4.0 or higher' % (weechat.prefix('error'),SCRIPT_NAME))
            weechat.command('','/wait 1ms /python unload %s' % SCRIPT_NAME)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019  Cole Helbling <cole.e.helbling@outlook.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# Changelog:
# 2019-12-14, Cole Helbling <cole.e.helbling@outlook.com>
#   version 1.0: initial release

SCRIPT_NAME = "styurl"
SCRIPT_AUTHOR = "Cole Helbling <cole.e.helbling@outlook.com>"
SCRIPT_VERSION = "1.0"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Style URLs with a Python regex"

import_ok = True
try:
    import weechat as w
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: https://weechat.org")
    import_ok = False

try:
    import re
except ImportError as message:
    print("Missing package for %s: %s" % (SCRIPT_NAME, message))
    import_ok = False

# https://mathiasbynens.be/demo/url-regex
# If you don't want to create your own regex, see the above link for options or
# ideas on creating your own

styurl_settings = {
    "buffer_type": (
        "formatted",
        "the type of buffers to run on (options are \"formatted\", \"free\", "
        "or \"*\" for both)"
    ),
    "format": (
        "${color:*_32}",
        "the style that should be applied to the URL"
        "(evaluated, see /help eval)"
    ),
    "ignored_buffers": (
        "core.weechat,python.grep",
        "comma-separated list of buffers to ignore URLs in "
        "(full name like \"irc.freenode.#alacritty\")"
    ),
    "ignored_tags": (
        "irc_quit,irc_join",
        "comma-separated list of tags to ignore URLs from"
    ),
    "regex": (
        r"((?:https?|ftp)://[^\s/$.?#].\S*)",
        "the URL-parsing regex using Python syntax "
        "(make sure capturing group 1 is the full URL)"
    ),
}

line_hook = None


def styurl_line_cb(data, line):
    """
    Callback called when a line is displayed.
    This parses the message for any URLs and styles them according to
    styurl_settings["format"].
    """
    global styurl_settings

    # Don't style the line if it's not going to be displayed... duh
    if line["displayed"] != "1":
        return line

    tags = line["tags"].split(',')
    ignored_tags = styurl_settings["ignored_tags"]

    # Ignore specified message tags
    if ignored_tags:
        if any(tag in tags for tag in ignored_tags.split(',')):
            return line

    bufname = line["buffer_name"]
    ignored_buffers = styurl_settings["ignored_buffers"]

    # Ignore specified buffers
    if ignored_buffers and bufname in ignored_buffers.split(','):
        return line

    message = line["message"]

    # TODO: enforce presence of a properly-formatted color object at
    # styurl_settings["format"] (eval object would also be valid, if it eval'd
    # to a color)

    regex = re.compile(styurl_settings["regex"])
    url_style = w.string_eval_expression(styurl_settings["format"], {}, {}, {})
    reset = w.color("reset")

    # Search for URLs and surround them with the defined URL styling
    formatted = regex.sub(r"%s\1%s" % (url_style, reset), message)
    line["message"] = line["message"].replace(message, formatted)

    return line


def styurl_config_cb(data, option, value):
    """Callback called when a script option is changed."""
    global styurl_settings, line_hook

    pos = option.rfind('.')
    if pos > 0:
        name = option[pos+1:]
        if name in styurl_settings:
            # Changing the buffer target requires us to re-hook to prevent
            # obsolete buffer types from getting styled
            if name == "buffer_type":
                if value in ("free", "formatted", "*"):
                    w.unhook(line_hook)
                    line_hook = w.hook_line(value, "", "", "styurl_line_cb",
                                            "")
                else:
                    # Don't change buffer type if it is invalid
                    w.prnt("", SCRIPT_NAME + ": Invalid buffer type: '%s', "
                           "not changing." % value)
                    w.config_set_plugin(name, styurl_settings[name])
                    return w.WEECHAT_RC_ERROR

            styurl_settings[name] = value

    return w.WEECHAT_RC_OK


def styurl_unload_cb():
    """Callback called when the script is unloaded."""
    global line_hook

    w.unhook(line_hook)
    del line_hook
    return w.WEECHAT_RC_OK


if __name__ == "__main__" and import_ok:
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                  SCRIPT_DESC, "styurl_unload_cb", ""):

        version = w.info_get("version_number", "") or 0

        for option, value in styurl_settings.items():
            if w.config_is_set_plugin(option):
                styurl_settings[option] = w.config_get_plugin(option)
            else:
                w.config_set_plugin(option, value[0])
                styurl_settings[option] = value[0]
            if int(version) >= 0x00030500:
                w.config_set_desc_plugin(option, "%s (default: \"%s\")"
                                         % (value[1], value[0]))

        w.hook_config("plugins.var.python." + SCRIPT_NAME + ".*",
                      "styurl_config_cb", "")

        # Style URLs
        line_hook = w.hook_line(styurl_settings["buffer_type"], "", "",
                                "styurl_line_cb", "")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 Germain Z. <germanosz@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Inspired by Emacs's view-lossage, this script displays a history of the last
# keystrokes you performed and the commands invoked.
#


import collections
import dataclasses
import re
from typing import Deque, Dict, Iterable, Tuple

import weechat  # type: ignore # pylint: disable=import-error


SCRIPT_NAME = "lossage"
SCRIPT_AUTHOR = "GermainZ <germanosz@gmail.com>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = (
    "Displays the last few input keystrokes and the commands run, "
    "inspired by Emacs's view-lossage."
)


HISTORY_SIZE: int = 300
REGEX_COMBO_REPL: Iterable[Tuple[re.Pattern, str]] = (
    (re.compile(r"\x01\[\["), "meta2-"),
    (re.compile(r"\x01\["), "meta-"),
    (re.compile(r"\x01"), "ctrl-"),
)
REGEX_AREA_STRIP = re.compile(r"^@[^:]+:")
HEADER: Tuple[str, str, str] = ("context", "combo", "command")


@dataclasses.dataclass
class HistoryItem:
    context: str
    combo: str
    command: str


@dataclasses.dataclass
class Data:
    history: Deque[HistoryItem] = dataclasses.field(
        default_factory=lambda: collections.deque(maxlen=HISTORY_SIZE)
    )
    key_bindings: Dict[str, Dict[str, str]] = dataclasses.field(
        default_factory=lambda: collections.defaultdict(dict)
    )


DATA = Data()


def cb_key_combo(context: str, signal: str, signal_data: str):
    mode = signal.split("_")[-1]
    combo = signal_data

    for regex, repl in REGEX_COMBO_REPL:
        combo = regex.sub(repl, combo)

    command = DATA.key_bindings[context].get(combo, "")
    DATA.history.append(HistoryItem(mode, combo, command))
    return weechat.WEECHAT_RC_OK


def cb_lossage_cmd(*_):
    buffer = weechat.buffer_search("python", SCRIPT_NAME)
    if not buffer:
        buffer = weechat.buffer_new(SCRIPT_NAME, "", "", "", "")
        weechat.buffer_set(buffer, "localvar_set_no_log", "1")
        weechat.buffer_set(buffer, "time_for_each_line", "0")
        weechat.buffer_set(buffer, "nicklist", "0")
    weechat.command(buffer, f"/buffer {SCRIPT_NAME}")
    weechat.command(buffer, "/buffer clear")

    weechat.prnt(
        buffer,
        f"{weechat.color('bold')}{HEADER[0]:10} {HEADER[1]:20} {HEADER[2]}",
    )
    for item in DATA.history:
        if item is None:
            break
        weechat.prnt(
            buffer, f"{item.context:10} {item.combo:20} {item.command}"
        )

    return weechat.WEECHAT_RC_OK


def cb_key_bindings_changed(*_):
    populate_key_bindings()


def populate_key_bindings():
    for context in ("default", "search", "cursor"):
        infolist = weechat.infolist_get("key", "", context)

        while weechat.infolist_next(infolist):
            key = weechat.infolist_string(infolist, "key")
            command = weechat.infolist_string(infolist, "command")

            # In the cursor context, bindings of the form `@area:key` can be
            # created. When the binding is used, the area is not passed to the
            # key_combo_* signal, so the best we can do is print a helpful
            # warning if there are several possible matches.
            if context == "cursor":
                key = REGEX_AREA_STRIP.sub("", key)
                if DATA.key_bindings[context].get(key, None):
                    command = (
                        f"ambiguous; see `/key list {context}` -> "
                        f"`@…:{key}` key bindings"
                    )

            DATA.key_bindings[context][key] = command

        weechat.infolist_free(infolist)


if __name__ == "__main__":
    weechat.register(
        SCRIPT_NAME,
        SCRIPT_AUTHOR,
        SCRIPT_VERSION,
        SCRIPT_LICENSE,
        SCRIPT_DESC,
        "",
        "",
    )

    populate_key_bindings()

    weechat.hook_signal("key_combo_default", "cb_key_combo", "default")
    weechat.hook_signal("key_combo_search", "cb_key_combo", "search")
    weechat.hook_signal("key_combo_cursor", "cb_key_combo", "cursor")
    weechat.hook_command(
        "lossage", SCRIPT_DESC, "", "test", "", "cb_lossage_cmd", ""
    )
    weechat.hook_signal("key_bind", "cb_key_bindings_changed", "")
    weechat.hook_signal("key_unbind", "cb_key_bindings_changed", "")



SCRIPT_NAME    = "xdccq"
SCRIPT_AUTHOR  = "Randall Flagg <shinigami_flagg@yahoo.it>"
SCRIPT_VERSION = "0.1.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Queue Xdcc messages to bots"

import_ok = True

try:
    import weechat
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False

botname = ""
pack = ""
# create a dictionary to save botnames and packs
# botname = {"botname1":"pack1","botname2":"pack2"}
# print myDict["jeff"] # => "jeffs value"
# print myDict.keys() # => ["john", "jeff"]

channel = ""


def xdccq_help_cb(data, buffer, args):
    """Callback for /xdccq command."""
    global botname, pack, channel
    response = {
        'add', 'list', 'listall', 'clear', 'clearall',
    }
    if args:
        words = args.strip().split(' ')
        if words[0] in response:
            if words[0] == "add":
                channel = buffer
                botname = words[1]
                pack = numToList(words[2])
                # look for packs aldready added
                # if already in transfer just add to list
                # else add and start transfer

                # check if bot is in auto accept nicks
                autonicks = weechat.config_string(weechat.config_get("xfer.file.auto_accept_nicks")).split(",")

                if not botname in autonicks:
                    xfer_option = weechat.config_get("xfer.file.auto_accept_nicks")
                    newlist = weechat.config_string(xfer_option)+","+botname

                    rc = weechat.config_option_set(xfer_option, newlist, 1)
                    if rc == weechat.WEECHAT_CONFIG_OPTION_SET_OK_CHANGED:
                        weechat.prnt('', "%s added to xdcc auto-accept list" % botname)
                    elif rc == weechat.WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE:
                        weechat.prnt('', "%s already in xdcc auto-accept list" % botname)
                    elif rc == weechat.WEECHAT_CONFIG_OPTION_SET_ERROR:
                        weechat.prnt('', "Error in adding %s in auto-accept list" % botname)
                else:
                    weechat.prnt('', "%s already in xdcc auto-accept nicks, not added." % botname)

                if len(pack):
                    runcommands()
                    pass
            elif words[0] == "list":
                # if botname[words[1]]:
                #     weechat.prnt('',"%s packs left" % botname[words[1]])
                #     weechat.prnt('',"from %s bot" % words[1])
                # else:
                #     weechat.prnt('',"Botname not in queue. Can't list!")
                pass
            elif words[0] == "listall":
                if len(pack):
                    weechat.prnt('', "%s packs left" % pack)
                    weechat.prnt('', "from %s bot" % botname)
                else:
                    weechat.prnt('', "No packs left")
            elif words[0] == "clear":
                # if botname[words[1]]:
                #     del botname[words[1]]
                #     weechat.prnt('',"%s bot queue cleared" % words[1])
                # else:
                #     weechat.prnt('',"Botname not in queue. Can't clear!")
                pass
            elif words[0] == "clearall":
                botname = ""
                pack = ""
                # botname.clear()
                weechat.prnt('', "Queue cleared")
        else:
            weechat.prnt('', "xdccq error: %s not a recognized command. Try /help xdccq" % words[0])

    return weechat.WEECHAT_RC_OK


def numToList(string):
    """Converts a string like '3,5,7-9,14' into a list."""
    ret = []
    numsplit = string.split(",")
    # the following code makes nums into a list of all integers
    for n in numsplit:
        nr = n.split('-')
        # handle the case of a single number
        if len(nr) == 1:
            try:
                ret.append(int(n))
            except:
                raise ValueError("number")
        # handle the case of a range
        elif len(nr) == 2:
            try:
                low = int(nr[0])
                nx = nr[1].split("%", 1)
                if len(nx) == 1:
                    high = int(nr[1]) + 1
                    step = 1
                else:
                    high = int(nx[0]) + 1
                    step = int(nx[1])
                if low > high:
                    raise ValueError("number")
                ret += list(range(low, high, step))
            except ValueError:
                raise ValueError("number")
        else:
            raise ValueError("range")
    return ret


def runcommands():
    global botname, pack, channel
    weechat.prnt('', "Pack %s remaining" % pack)
    if len(pack):
        onepack = pack.pop(0)
        weechat.command(channel, "/msg " + botname + " xdcc send " + str(onepack))
    return weechat.WEECHAT_RC_OK


def xfer_ended_signal_cb(data, signal, signal_data):
    # at the end of transfer print the botname and completed file
    # weechat.infolist_next(signal_data)
    # weechat.prnt('',"%s" % weechat.infolist_string(signal_data, 'remote_nick'))
    runcommands()
    return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
    weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", "")
    weechat.hook_command(
            SCRIPT_NAME, SCRIPT_DESC,
            '\nadd [name] packs\n list\n listall [name]\n clear\n clearall [name]',
            'ADD: adds packs to [botname] queue  \n LIST: list [botname] queue \n Pack format can be 1-10 or 1,2,3 or 1-10,12,15 \n LISTALL: list all queue \n CLEAR: clean all queues \n CLEARALL: clears queue for [botname]',
            'add %(nick) packs'
            ' || list  %(nick)'
            ' || listall'
            ' || clear %(nick)'
            ' || clearall',
            'xdccq_help_cb', '')
    weechat.hook_signal("xfer_ended", "xfer_ended_signal_cb", "")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2009 xt <xt@bash.no>
# Copyright (C) 2011 quazgaa <quazgaa@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#-------------------------------------------------------------------
#  To make bandwidth monitor visible you need to put "[bandwidth]"
#  (without "") in your weechat.bar.status.items setting
#-------------------------------------------------------------------

#
# History:
#
# 2020-09-05, mumixam
#     version 1.1: add python3 support while still supporting python2
# 2011-12-02, quazgaa <quazgaa@gmail.com>
#     version 1.0: Complete rewrite.  Make script more featureful, robust, and accurate.
#                  Thanks to FlashCode and ze for helping debug.
# 2011-11-29, quazgaa <quazgaa@gmail.com>
#     version 0.2.1: fixed: take refresh_rate into account for bandwidth calculation
# 2009-10-15, xt <xt@bash.no>:
#     version 0.2: error checking from output command
# 2009-10-14, xt <xt@bash.no>:
#     version 0.1: initial release inspired by nils' perl script
#


# this is a weechat script
try:
    import weechat
except:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    raise SystemExit(0)

try:
    from time import time
except:
    print("Error importing time module.")
    raise SystemExit(0)

try:
    unichr
except NameError:
    unichr = chr

# defines
SCRIPT_NAME     = "bandwidth"
SCRIPT_AUTHOR   = "xt <xt@bash.no>"
SCRIPT_VERSION  = "1.1"
SCRIPT_LICENSE  = "GPL3"
SCRIPT_DESC     = "Displays network interface bandwidth (KiB/s and MiB/s) on a bar"
SCRIPT_SETTINGS = {
    "device"         : ("eth0",                           "Network interface(s) to monitor, in order, separated by ';'"),
    "refresh_rate"   : ("5",                              "Refresh rate in seconds"),
    "format"         : (("%N(" + unichr(8595) + "%DV%DU/s " + unichr(8593) + "%UV%UU/s)"),
                        "Output formatting: %N = network interface, %DV = downstream value, %DU = downstream units (K or M), %UV = upstream value, %UU = upstream units (K or M).  Note: default setting uses UTF-8"),
    "separator"      : (" ",                              "String displayed between output for multiple devices"),
}
STATS_FILE      = "/proc/net/dev"


# global variables
last_device = []
last_down_bytes = []
last_up_bytes = []
last_time = 0


def main():
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        version = int(weechat.info_get('version_number', '')) or 0

        # unset unused setting from older versions of script
        if weechat.config_is_set_plugin('display_unit'):
            weechat.prnt("", "Option plugins.var.python.bandwidth.display_unit no longer used, removing.")
            weechat.config_unset_plugin('display_unit')

        # set default settings
        for option in SCRIPT_SETTINGS.keys():
            if not weechat.config_is_set_plugin(option):
                value = SCRIPT_SETTINGS[option][0]
                if isinstance(value, str):
                    pass
                elif isinstance(value, bytes):
                    pass
                elif isinstance(value, unicode):
                    value = value.encode('utf8')
                weechat.config_set_plugin(option, value)
            if version >= 0x00030500:
                weechat.config_set_desc_plugin(option, SCRIPT_SETTINGS[option][1])

        # ensure sane refresh_rate setting
        if int(weechat.config_get_plugin('refresh_rate')) < 1:
            weechat.prnt("", "{}Invalid value for option plugins.var.python.bandwidth.refresh_rate, setting to default of {}".format(weechat.prefix("error"), SCRIPT_SETTINGS['refresh_rate'][0]))
            weechat.config_set_plugin('refresh_rate', SCRIPT_SETTINGS['refresh_rate'][0])

        # create the bandwidth monitor bar item
        weechat.bar_item_new('bandwidth', 'bandwidth_item_cb', '')
        # update it every plugins.var.python.bandwidth.refresh_rate seconds
        weechat.hook_timer(int(weechat.config_get_plugin('refresh_rate'))*1000, 0, 0, 'bandwidth_timer_cb', '')


def bandwidth_timer_cb(data, remaining_calls):
    weechat.bar_item_update('bandwidth')
    return weechat.WEECHAT_RC_OK


def bandwidth_item_cb(data, buffer, args):
    global last_device, last_down_bytes, last_up_bytes, last_time

    device = weechat.config_get_plugin('device').strip(';').split(';')
    output_format = weechat.config_get_plugin('format')
    separator = weechat.config_get_plugin('separator')
    invalid_settings = False

    # ensure sane settings
    if not device[0]:
        weechat.prnt("", "{}Option plugins.var.python.bandwidth.device should contain at least one device name, setting to default of {}".format(weechat.prefix("error"), SCRIPT_SETTINGS['device'][0]))
        weechat.config_set_plugin('device', SCRIPT_SETTINGS['device'][0])
        invalid_settings = True
    if int(weechat.config_get_plugin('refresh_rate')) < 1:
        weechat.prnt("", "{}Invalid value for option plugins.var.python.bandwidth.refresh_rate, setting to default of {}".format(weechat.prefix("error"), SCRIPT_SETTINGS['refresh_rate'][0]))
        weechat.config_set_plugin('refresh_rate', SCRIPT_SETTINGS['refresh_rate'][0])
        invalid_settings = True
    if '%DV' not in output_format and '%UV' not in output_format:
        weechat.prnt("", "{}Option plugins.var.python.bandwidth.format should contain at least one of: '%DV' or '%UV'. Setting to default of '{}'".format(weechat.prefix("error"), SCRIPT_SETTINGS['format'][0]))
        weechat.config_set_plugin('format', SCRIPT_SETTINGS['format'][0])
        invalid_settings = True
    if invalid_settings:
        return ''

    # open the network device status information file
    try:
        f = open(STATS_FILE)
    except:
        weechat.prnt("", "{}Error opening {}".format(weechat.prefix("error"), STATS_FILE))
        return ''
    else:
        current_time = time()
        try:
            foo = f.read()
        except:
            weechat.prnt("", "{}Error reading {}".format(weechat.prefix("error"), STATS_FILE))
            f.close()
            return ''
        f.close()

    current_down_bytes = []
    current_up_bytes = []
    num_devices = len(device)
    num_last_devices = len(last_device)
    lines = foo.splitlines()
    new_device_list = False
    device_exist = False

    # get the downstream and upstream byte counts
    for i in range(num_devices):
        for line in lines:
            if (device[i] + ':') in line:
                field = line.split(':')[1].strip().split()
                current_down_bytes.append(float(field[0]))
                current_up_bytes.append(float(field[8]))
                device_exist = True
                break
        if device_exist:
            device_exist = False
        else:
            current_down_bytes.append(0)
            current_up_bytes.append(0)

    # check if the set of network devices to monitor has changed while script is running,
    if last_device:
        if num_last_devices != num_devices:
            new_device_list = True
        else:
            for i in range(num_devices):
                if device[i] != last_device[i]:
                    new_device_list = True
                    break

    # if so, clear the global variables,
    if new_device_list:
        del last_device[:]
        del last_down_bytes[:]
        del last_up_bytes[:]

    # set them afresh (also if script first starting),
    if not last_device:
        if num_devices:
            for i in range(num_devices):
                last_device.append(device[i])
                last_down_bytes.append(current_down_bytes[i])
                last_up_bytes.append(current_up_bytes[i])
            last_time = current_time
        # and start from the beginning
        return ''

    # calculate downstream and upstream rates in KiB/s
    if num_devices:
        down_rate = []
        up_rate = []
        time_elapsed = current_time - last_time
        last_time = current_time

        for i in range(num_devices):
            down_rate.append((current_down_bytes[i] - last_down_bytes[i]) / time_elapsed / 1024)
            up_rate.append((current_up_bytes[i] - last_up_bytes[i]) / time_elapsed / 1024)
            last_down_bytes[i] = current_down_bytes[i]
            last_up_bytes[i] = current_up_bytes[i]

        output_item = [output_format for i in device]

    output = ''

    # determine downstream and upstream units; format the output
    for i in range(num_devices):
        if '%DU' in output_item[i]:
            if down_rate[i] >= 1024:
                down_rate[i] = round((down_rate[i]/1024), 1)
                down_rate_unit = 'M'
            else:
                down_rate[i] = int(round(down_rate[i]))
                down_rate_unit = 'K'
            output_item[i] = output_item[i].replace('%DU', down_rate_unit)

        if '%UU' in output_item[i]:
            if up_rate[i] >= 1024:
                up_rate[i] = round((up_rate[i]/1024), 1)
                up_rate_unit = 'M'
            else:
                up_rate[i] = int(round(up_rate[i]))
                up_rate_unit = 'K'
            output_item[i] = output_item[i].replace('%UU', up_rate_unit)

        output_item[i] = output_item[i].replace('%DV', str(down_rate[i]))
        output_item[i] = output_item[i].replace('%UV', str(up_rate[i]))
        output_item[i] = output_item[i].replace('%N', device[i])

        if output:
            output += separator + output_item[i]
        else:
            output = output_item[i]

    # return the result
    return output

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ex:sw=4 ts=4:ai:
#
# SPDX-FileCopyrightText: 2012 Krister Svanlund <krister.svanlund@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#   based on tcl version:
#    Remote Notification Script v1.1
#    by Gotisch <gotisch@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# Example usage when Weechat is running on a remote PC and you want
# want to use port 4321 for the connection.
#
#     On the "client" (where the notifications will end up), host is
#     the remote host where weechat is running:
#       python2 location/of/pyrnotify.py 4321 & ssh -R 4321:localhost:4321 username@host
#     You can have a second argument to specified the time to display the notification
#       python2 location/of/pyrnotify.py 4321 2000 & ssh -R 4321:localhost:4321 username@host
#     Important to remember is that you should probably setup the
#     connection with public key encryption and use something like
#     autossh to do this in the background.
#
#     In weechat:
#       /python load pyrnotify.py
#       and set the port
#       /set plugins.var.python.pyrnotify.port 4321
#
# It is also possible to set which host pyrnotify shall connect to,
# this is not recommended. Using a ssh port-forward is much safer
# and doesn't require any ports but ssh to be open.

# ChangeLog:
# 2025-12-06: Modernize code, avoid escaping using regex by using json
#             instead of shell-like serialization.
#
# 2018-08-20: Make it work with python3
#              use of sendall instead of send
# 2014-05-10: Change hook_print callback argument type of displayed/highlight
#             (WeeChat >= 1.0)
# 2012-06-19: Added simple escaping to the title and body strings for
#             the script to handle trailing backslashes.


try:
    import weechat as w

    in_weechat = True
except ImportError as e:
    in_weechat = False

import json
import os
import re
import socket
import subprocess
import sys

SCRIPT_NAME = "pyrnotify"
SCRIPT_AUTHOR = "Krister Svanlund <krister.svanlund@gmail.com>"
SCRIPT_VERSION = "2.0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Send remote notifications over SSH"


def run_notify(icon, nick, chan, message):
    host = w.config_get_plugin("host")
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, int(w.config_get_plugin("port"))))
        msg = {
            "urgency": "normal",
            "icon": icon,
            "nick": nick,
            "chan": chan,
            "message": message,
        }
        s.sendall(json.dumps(msg, ensure_ascii=False).encode("UTF-8"))
        s.close()
    except Exception as e:
        w.prnt("", "Could not send notification: {0}".format(e))


def on_msg(*a):
    if len(a) == 8:
        data, buffer, timestamp, tags, displayed, highlight, sender, message = a
        if data == "private" or int(highlight):
            if data == "private" and w.config_get_plugin("pm-icon"):
                icon = w.config_get_plugin("pm-icon")
            else:
                icon = w.config_get_plugin("icon")
            buffer = (
                "me" if data == "private" else w.buffer_get_string(buffer, "short_name")
            )
            run_notify(icon, sender, buffer, message)
    return w.WEECHAT_RC_OK


def weechat_script():
    settings = {
        "host": "localhost",
        "port": "4321",
        "icon": "utilities-terminal",
        "pm-icon": "emblem-favorite",
    }
    if w.register(
        SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""
    ):
        for kw, v in settings.items():
            if not w.config_get_plugin(kw):
                w.config_set_plugin(kw, v)
        w.hook_print("", "notify_message", "", 1, "on_msg", "")
        w.hook_print("", "notify_private", "", 1, "on_msg", "private")
        w.hook_print(
            "", "notify_highlight", "", 1, "on_msg", ""
        )  # Not sure if this is needed


######################################
## This is where the client starts, except for the global if-check nothing below this line is
## supposed to be executed in weechat, instead it runs when the script is executed from
## commandline.


def accept_connections(s, timeout=None):
    conn, addr = s.accept()
    try:
        data = b""
        while d := conn.recv(1024):
            data += d
    finally:
        conn.close()
    if data:
        try:
            notif = json.loads(data.decode("UTF-8"))
            argv = [
                "notify-send",
                "-u",
                notif["urgency"],
                "-c",
                "IRC",
                "-i",
                notif["icon"],
                "--",
                notif["chan"],
                notif["message"],
            ]
            if timeout:
                argv.extend(["-t", timeout])
            subprocess.run(argv)
        except (ValueError, OSError) as e:
            print(e)


def weechat_client(argv):
    port = int(argv[1]) if len(sys.argv) > 1 else 4321
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(("localhost", port))
    s.listen(5)
    try:
        while True:
            accept_connections(s, argv[2] if len(sys.argv) > 2 else None)
    except KeyboardInterrupt:
        return
    finally:
        s.close()


if __name__ == "__main__":
    if in_weechat:
        weechat_script()
    else:
        weechat_client(sys.argv)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012-2017 by nils_2 <weechatter@arcor.de>
#
# quickly add/del/change entry in nick_color_force
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2017-08-17: nils_2,(freenode.#weechat)
#      0.6.1: print nicks in sorted order
# 2017-05-18: ticalc-travis (https://github.com/weechatter/weechat-scripts/pull/18)
#       0.6 : Clean up some redundant code
#           : Add nicks to irc.look.nick_color_force in sorted order for easier manual editing
#           : Display proper feedback on incorrect commands
#           : Fix inconsistencies in help syntax
#           : Don't retain nicks that have been manually removed from nick_color_force
#           : Provide feedback messages for successful operations
# 2016-04-17: nils_2,(freenode.#weechat)
#       0.5 : make script compatible with option weechat.look.nick_color_force (weechat >=1.5)
# 2013-01-25: nils_2,(freenode.#weechat)
#       0.4 : make script compatible with Python 3.x
# 2012-07-08: obiwahn, (freenode)
#     0.3.1 : fix: list nick
#           : - show nick: color if it is in list
#           :   else tell color for nick is not set
# 2012-05-23: unferth, (freenode.#weechat)
#       0.3 : add: show current colors
# 2012-02-14: nils_2, (freenode.#weechat)
#       0.2 : fix: problem with foreground/background color
#           : add: show only a given nick
# 2012-02-01: nils_2, (freenode.#weechat)
#       0.1 : initial release
#
# requires: WeeChat version 0.3.4
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts
#

try:
    import weechat, re

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = "quick_force_color"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "0.6.1"
SCRIPT_LICENSE  = "GPL3"
SCRIPT_DESC     = "quickly add/del/change entry in nick_color_force"

# weechat standard colours.
DEFAULT_COLORS = {  0 : "darkgray", 1 : "red", 2 : "lightred", 3 : "green",
                    4 : "lightgreen", 5 : "brown", 6 : "yellow", 7 : "blue",
                    8 : "lightblue", 9 : "magenta", 10 : "lightmagenta", 11 : "cyan",
                   12 : "lightcyan", 13 : "white"}

colored_nicks = {}
nick_option_old = "irc.look.nick_color_force"
nick_option_new = "weechat.look.nick_color_force"
nick_option = ""
# ================================[ callback ]===============================
def print_usage(buffer):
    weechat.prnt(buffer, "Usage: /%s list [nick] | add nick color | del nick" % SCRIPT_NAME)

def nick_colors_cmd_cb(data, buffer, args):
    global colored_nicks

    if args == "":                                                                              # no args given. quit
        print_usage(buffer)
        return weechat.WEECHAT_RC_OK

    argv = args.strip().split(" ")
    if (len(argv) == 0) or (len(argv) >= 4):                                                    # maximum of 3 args!!
        print_usage(buffer)
        return weechat.WEECHAT_RC_OK

    bufpointer = weechat.window_get_pointer(buffer,'buffer')                                    # current buffer

    create_list()

    if argv[0].lower() == 'list':                                                               # list all nicks
        if len(colored_nicks) == 0:
            weechat.prnt(buffer,'%sno nicks in \"%s\"...' % (weechat.prefix("error"),nick_option))
        elif len(argv) == 2:
            if argv[1] in colored_nicks:
                color = colored_nicks[argv[1]]                                                  # get color from given nick
                weechat.prnt(buffer,"%s%s: %s" % (weechat.color(color),argv[1],color))
            else:
                weechat.prnt(buffer,"no color set for: %s" % (argv[1]))

        else:
            weechat.prnt(buffer,"List of nicks in : %s" % nick_option)

            for nick,color in sorted(list(colored_nicks.items())):
                weechat.prnt(buffer,"%s%s: %s" % (weechat.color(color),nick,color))

    elif (argv[0].lower() == 'add') and (len(argv) == 3):
        if argv[1] in colored_nicks:
            weechat.prnt(buffer, "Changing nick '%s' to color %s%s" % (argv[1], weechat.color(argv[2]), argv[2]))
        else:
            weechat.prnt(buffer, "Adding nick '%s' with color %s%s" % (argv[1], weechat.color(argv[2]), argv[2]))
        colored_nicks[argv[1]] = argv[2]
        save_new_force_nicks()

    elif (argv[0].lower() == 'del') and (len(argv) == 2):
        if argv[1] in colored_nicks:                                                            # search if nick exists
            del colored_nicks[argv[1]]
            save_new_force_nicks()
            weechat.prnt(buffer, "Removed nick '%s'" % argv[1])
        else:
            weechat.prnt(buffer, "Nick '%s' not found in nick_color_force" % argv[1])
    else:
        print_usage(buffer)

    return weechat.WEECHAT_RC_OK

def save_new_force_nicks():
    global colored_nicks
    new_nick_color_force = ';'.join([ ':'.join(item) for item in sorted(colored_nicks.items())])
    config_pnt = weechat.config_get(nick_option)
    weechat.config_option_set(config_pnt,new_nick_color_force,1)

def nick_colors_completion_cb(data, completion_item, buffer, completion):
#    for id,color in DEFAULT_COLORS.items():
    for id,color in list(DEFAULT_COLORS.items()):
        weechat.hook_completion_list_add(completion, color, 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def force_nick_colors_completion_cb(data, completion_item, buffer, completion):
    create_list()
#    for nick,color in colored_nicks.items():
    for nick,color in list(colored_nicks.items()):
        weechat.hook_completion_list_add(completion, nick, 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def create_list():
    global nick_color_force,colored_nicks
#        colored_nicks = dict([elem.split(':') for elem in nick_color_force.split(';')])
    colored_nicks = {}
    nick_color_force = weechat.config_string(weechat.config_get(nick_option))                   # get list
    if nick_color_force != '':
        nick_color_force = nick_color_force.strip(';')                                          # remove ';' at beginning and end of string
        for elem in nick_color_force.split(';'):                                                # split nick1:color;nick2:color
            counter = elem.count(':')
            if counter == 1:
                nick,colors = elem.split(':')                                                   # nick1:color_fg,color_bg
                colored_nicks.setdefault(nick,colors)
            elif counter == 2:
                nick,color_fg,color_bg = elem.split(':')                                        # nick1:color_fg:color_bg
                colored_nicks.setdefault(nick,color_fg+':'+color_bg)

# ================================[ main ]===============================
if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get('version_number', '') or 0
        if int(version) >= 0x00030400:
            weechat.hook_command(SCRIPT_NAME,SCRIPT_DESC,
            'add <nick> <color> || del <nick> <color> || list [<nick>]',
            'add <nick> <color>: add a nick with its color to nick_color_force\n'
            'del <nick>        : delete given nick with its color from nick_color_force\n'
            'list [<nick>]     : list all forced nicks with its assigned color or optional from one nick\n\n'
            'Examples:\n'
            ' add nick nils_2 with color red:\n'
            '  /' + SCRIPT_NAME + ' add nils_2 red\n'
            ' recolor nick nils_2 with foreground color yellow and background color blue:\n'
            '  /' + SCRIPT_NAME + ' add nils_2 yellow:blue\n'
            ' delete nick nils_2:\n'
            '  /' + SCRIPT_NAME + ' del nils_2\n',
            'add %(nick) %(plugin_nick_colors) %-||'
            'del %(plugin_force_nick) %-||'
            'list %(plugin_force_nick) %-',
            'nick_colors_cmd_cb', '')
            nick_option = nick_option_old
            if int(version) >= 0x01050000:
                nick_option = nick_option_new
            weechat.hook_completion('plugin_nick_colors', 'nick_colors_completion', 'nick_colors_completion_cb', '')
            weechat.hook_completion('plugin_force_nick', 'force_nick_colors_completion', 'force_nick_colors_completion_cb', '')
        else:
            weechat.prnt("","%s%s %s" % (weechat.prefix("error"),SCRIPT_NAME,": needs version 0.3.4 or higher"))
